# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################                 Aero Data Plugin                 #############################
############################################################################################################

# Abstract:
This Paraview's programmable filter takes 1 input from pipeline: a 2D/3D data (CFD or experimental data) and 
completes the remaining aerodynamic field parameters if necessary. Moreover, it calculates far-field-method 
and exergy-method parameters at all the points. Details of the formulations and how to use the plugin are 
available in the User's Manual.


# Disclaimer:
This plugin was developed for research purposes and released under GNU GPL V3 license without any guarantee
of any kind, either express or implied. Its use for teaching, research and industrial applications is allowed
under the strict responsibility of the user. The authors of this plugin are not responsible for any damages
or errors related to the use of the plugin.
"""


print "#########################################################################################################################################"
print "##################################                                                                     ##################################"
print "##################################                              EPSILON 2.0                            ##################################"
print "##################################                        An exergy analysis tool                      ##################################"
print "##################################                                                                     ##################################"
print "#########################################################################################################################################"
print ""
print "                                             Developed at DAEP/ISAE-SUPAERO, France, 2018 "
print ""
print "#DISCLAIMER:"
print "This plugin was developed for research purposes and released under GNU GPL V3 license without any guarantee of any kind, either express"
print "or implied. Its use for teaching, research and industrial applications is allowed under the strict responsibility of the user. The authors"
print "of this plugin are not responsible for any damages or errors related to the use of the plugin."

print ""
print ""
print ""
print "########################################################"
print "##########             OUTPUT INFO            ##########"
print "########################################################"


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import numpy as np
import vtk
from vtk.numpy_interface import algorithms as algs



# Here we define the path to the readers files for the different formats:
import sys
sys.path.insert(0, r'C:/EPSILON/Readers')


from Reader_elsA_Tecplot_Binary import *
from Reader_elsA_CGNS_BC import *
from Reader_Epsilon_Wind_Tunnel import *
from Reader_Fluent_CAS import * 
from Reader_Fluent_CGNS import * 
from Reader_Fluent_Ensight import *
from Reader_Open_Foam import *
from Reader_Potential_Flow_Generator import *
from Reader_STAR_CCM_Ensight import *
from Reader_TAU_Tecplot import *
from Reader_New_Format import *


############################################################################################################
############################################# INPUT PARAMETERS #############################################
############################################################################################################

# Input parameters are defined as string in Paraview's GUI, thus a string to float convertion is required:
InputFormat = float(InputFormat)
UpwardAxis = float(UpwardAxis)      
Alfa = float(Alfa) 
Beta = float(Beta)
SpeedInputType = float(SpeedInputType)
Speed = float(Speed)
TypeOfParameter = float(TypeOfParameter)
P0 = float(P0)
T0 = float(T0)
Sref = float(Sref)
Nondimensionalize = float(Nondimensionalize)
AERO = float(AERO)
NEAR_FIELD = float(NEAR_FIELD)
FAR_FIELD = float(FAR_FIELD)
EXERGY = float(EXERGY)
LAMB_VECTOR = float(LAMB_VECTOR)
Dimension = float(Dimension)


############################################################################################################
######################################### COMPLETING UPSTREAM DATA #########################################
############################################################################################################

R = 287.04213969                                                        # Gas constant R [J/Kg*K]
Gamma = 1.399008873407                                                  # Gas constant Gamma
CpGas = (Gamma*R)/(Gamma-1)                                             # Gas constant Cp [J/Kg*K]
CvGas = CpGas-R                                                         # Gas constant Cv [J/Kg*K]


# Data completion procedure: it depends on SpeedInputType (Mach or velocity) and TypeOfParameter (Total or static) selected by the user
if SpeedInputType == 0 and TypeOfParameter == 0:
    M0 = Speed                                                          # Mach number
    Pt0 = P0                                                            # Total pressure [Pa]
    Tt0 = T0                                                            # Total temperature [K]
    Ps0 = Pt0/((1+0.5*(Gamma-1)*M0**2)**(Gamma/(Gamma-1)))              # Static pressure [Pa]
    Ts0 = Tt0/(1+0.5*(Gamma-1)*M0**2)                                   # Static temperature [K]
    e0 = CvGas*Ts0                                                      # Internal energy [J/Kg]
    Rho0 = Ps0/(R*Ts0)                                                  # Density [Kg/m^3]
    a0 = np.sqrt(Gamma*R*Ts0)                                           # Speed of sound [m/s]
    V0 = M0*a0                                                          # Airspeed [m/s]
    q0 = 0.5*Rho0*V0**2                                                 # Dynamic pressure [Pa]
    Hs0 = CpGas*Ts0                                                     # Static enthalpy [J/Kg]
    Ht0 = CpGas*Tt0                                                     # Total enthalpy [J/Kg]
    
elif SpeedInputType == 0 and TypeOfParameter == 1:
    M0 = Speed                                                          
    Ps0 = P0
    Ts0 = T0      
    Pt0 = Ps0*((1+0.5*(Gamma-1)*M0**2)**(Gamma/(Gamma-1)))              
    Tt0 = Ts0*(1+0.5*(Gamma-1)*M0**2)                                   
    e0 = CvGas*Ts0                                                      
    Rho0 = Ps0/(R*Ts0)                                                  
    a0 = np.sqrt(Gamma*R*Ts0)                                          
    V0 = M0*a0                                                          
    q0 = 0.5*Rho0*V0**2                                                 
    Hs0 = CpGas*Ts0
    Ht0 = CpGas*Tt0

elif SpeedInputType == 1 and TypeOfParameter == 0:
    V0 = Speed
    Pt0 = P0
    Tt0 = T0
    Ts0 = Tt0-((0.5*V0**2)/CpGas)
    e0 = CvGas*Ts0                                                      
    a0 = np.sqrt(Gamma*R*Ts0)                                           
    M0 = V0/a0     
    Ps0 = Pt0/(1+0.5*(Gamma-1)*M0**2)**(Gamma/(Gamma-1))
    Rho0 = Ps0/(R*Ts0)
    q0 = 0.5*Rho0*V0**2
    Hs0 = CpGas*Ts0
    Ht0 = CpGas*Tt0

elif SpeedInputType == 1 and TypeOfParameter == 1:
    V0 = Speed
    Ps0 = P0
    Ts0 = T0 
    Tt0 = Ts0+((0.5*V0**2)/CpGas)
    e0 = CvGas*Ts0                                                      
    a0 = np.sqrt(Gamma*R*Ts0)                                           
    M0 = V0/a0     
    Pt0 = Ps0*(1+0.5*(Gamma-1)*M0**2)**(Gamma/(Gamma-1))
    Rho0 = Ps0/(R*Ts0)
    q0 = 0.5*Rho0*V0**2
    Hs0 = CpGas*Ts0
    Ht0 = CpGas*Tt0
	
	
	
############################################################################################################
#################################### READING CFD/EXPERIMENTAL INPUT DATA ###################################
############################################################################################################

# Converting the input from Cell Data To Point Data:
myArrays=vtk.vtkPassArrays()                                 	# use pass arrays to extract a copy of the input data
myArrays.SetInputDataObject(self.GetInputDataObject(0,0))       # We call the Paraview internal port where input data is stored

c2p = vtk.vtkCellDataToPointData()     							# use Cell2Point to convert the arrays
c2p.SetInputConnection(myArrays.GetOutputPort())
c2p.Update()

INPUT = dsa.WrapDataObject(c2p.GetOutput()) 					# Get the point data: it replaces the original cell-based INPUT by a new point-based INPUT



# It executes the selected file format reader function, which launches another Python file. It reads the raw data comming from the INPUT file.
if InputFormat==0:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_elsA_Tecplot_Binary(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==1:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_elsA_CGNS_BC(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==2:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Epsilon_Wind_Tunnel(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0,Pt0,V0)
elif InputFormat==3:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Fluent_CAS(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==4:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Fluent_CGNS(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)  
elif InputFormat==5:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Fluent_Ensight(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)	
elif InputFormat==6:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Open_Foam(INPUT,R,CvGas,CpGas,Ps0,Pt0,Rho0,Ts0,V0,TypeOfParameter)
elif InputFormat==7:   																											
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_Potential_Flow_Generator(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==8:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_STAR_CCM_Ensight(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==9:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_TAU_Tecplot(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)
elif InputFormat==10:
    POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff = Reader_New_Format(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0)

	
############################################################################################################
########################################### COMPLETING FIELD DATA ##########################################
############################################################################################################



# Calculating the remaining parameters
Hs = CpGas*Ts															# Static enthalpy [J/Kg]
eint = CvGas*Ts                                                         # Internal energy [J/Kg]
a = algs.sqrt(Gamma*R*Ts)                                               # Speed of sound [m/s]
V = algs.sqrt(Vx**2+Vy**2+Vz**2)                                        # Speed magnitude [m/s]
M = V/a                                                                 # Mach number
q = 0.5*Rho*V**2                                                        # Dynamic pressure [Pa]
Cp = (Ps-Ps0)/q0                                                        # Pressure coeffcient
Pt = Ps*(1+0.5*(Gamma-1)*M**2)**(Gamma/(Gamma-1))                       # Total pressure [Pa]
Tt = Ts*(1+0.5*(Gamma-1)*M**2)                                          # Total temperature [K]
Ht = Hs+0.5*V**2                                                        # Total enthalpy [J/Kg]
DeltaHt = Ht-Ht0                                                        # Total enthalpy variation respect to upstream value [J/Kg]
DeltaS = CpGas*algs.log(Ts/Ts0)-R*algs.log(Ps/Ps0)                      # Specific entropy variation referred to upstream value [J/Kg*K]
PtPt0 = Pt/Pt0  														# Total pressure ratio

Vector_Speed = algs.make_vector(Vx,Vy,Vz)    							# Velocity vector in the mesh reference frame [m/s]
V_gradient = gradient(Vector_Speed)  									# Velocity gradient tensor in the mesh reference frame[1/s]

VORTICITY = algs.vorticity(Vector_Speed)       							# Vorticity vector in the mesh reference frame [1/s]
Helicity = algs.dot(Vector_Speed,VORTICITY)/algs.max(algs.abs(algs.dot(Vector_Speed,VORTICITY)))   #Normalized helicity

STRAIN = strain(Vector_Speed)											# Strain tensor [1/s] in the mesh reference frame
#Strain_Magnitude = np.sqrt(np.einsum('ijk,ijk->i',STRAIN,STRAIN))      # Magnitude of a tensor (L2_1 norm), kept here for debugging purposes

Ps_gradient = gradient(Ps)
Ts_gradient = gradient(Ts)

Shock_Detector = algs.dot(Vector_Speed,Ps_gradient)/(a*algs.mag(Ps_gradient)) 					# Shockwave detector parameter (Lovely and Haimes criterion)
Q_criterion = 0.5*(algs.mag(VORTICITY)**2-np.sqrt(np.einsum('ijk,ijk->i',STRAIN,STRAIN))**2)    # Vortex detector by Q-criterion. it uses the L2_1 norm of the strain tensor
Lambda_2 = eigenvalue(STRAIN**2+(V_gradient - STRAIN)**2)[:,1]									# Vortex detector by Lambda 2 criterion

Phi_eff = MU_EFF*(2*V_gradient[:,0,0]**2 + 2*V_gradient[:,1,1]**2 + 2*V_gradient[:,2,2]**2 + (V_gradient[:,0,1]+V_gradient[:,1,0])**2 + (V_gradient[:,0,2]+V_gradient[:,2,0])**2 + (V_gradient[:,2,1]+V_gradient[:,1,2])**2 - (2.0/3.0)*(V_gradient[:,0,0]+V_gradient[:,1,1]+V_gradient[:,2,2])**2)  # Effective dissipation         


############################################################################################################
########################################### ROTATING VECTOR DATA ###########################################
############################################################################################################

if UpwardAxis==0:    #Mesh Y-axis pointing upwards
	AngleX = -90
	AngleY = Beta
	AngleZ = Alfa  	
elif UpwardAxis==1:  #Mesh Z-axis pointing upwards
	AngleX = 0      
	AngleY = -Alfa
	AngleZ = Beta

Rx = np.array([(1,0,0),(0,algs.cos(AngleX*np.pi/180),-algs.sin(AngleX*np.pi/180)),(0,algs.sin(AngleX*np.pi/180),algs.cos(AngleX*np.pi/180))])   #Rotation matrix about X-axis
Ry = np.array([(algs.cos(AngleY*np.pi/180),0,algs.sin(AngleY*np.pi/180)),(0,1,0),(-algs.sin(AngleY*np.pi/180),0,algs.cos(AngleY*np.pi/180))])  	#Rotation matrix about Y-axis
Rz = np.array([(algs.cos(AngleZ*np.pi/180),-algs.sin(AngleZ*np.pi/180),0),(algs.sin(AngleZ*np.pi/180),algs.cos(AngleZ*np.pi/180),0),(0,0,1)])  	#Rotation matrix about Z-axis

Rx_T = np.transpose(Rx)
Ry_T = np.transpose(Ry)
Rz_T = np.transpose(Rz)

MX = np.dot(Rx_T,np.dot(Rx,Rx_T))  #Transformation matrix from the original reference system (mesh-XYZ) to the new reference system (aerodynamic-xyz) --> rotation about mesh X-axis
MY = np.dot(Ry_T,np.dot(Ry,Ry_T))  #Transformation matrix from the original reference system (mesh-XYZ) to the new reference system (aerodynamic-xyz) --> rotation about mesh Y-axis
MZ = np.dot(Rz_T,np.dot(Rz,Rz_T))  #Transformation matrix from the original reference system (mesh-XYZ) to the new reference system (aerodynamic-xyz) --> rotation about mesh Z-axis

TRANSFORMATION = np.dot(MX,np.dot(MY,MZ))  #Total (combined) transformation matrix


# Some drag formulations require the actual coordinate of the domain points expressed in the aerodynamic reference system:
points = np.einsum('ij,kj->ik',POINTS,TRANSFORMATION)     #Coordinates in the aerodynamic reference syste. "einsum" is a Pythonic fashion to do a dot product: np.dot(TRANSFORMATION,POINTS)
x = points[:,0]   #x-coordinate in the aerodynamic reference frame
y = points[:,1]   #y-coordinate in the aerodynamic reference frame
z = points[:,2]   #z-coordinate in the aerodynamic reference frame

velocity = np.einsum('ij,kj->ik',Vector_Speed,TRANSFORMATION) 		# Velocity vector in the aerodynamic reference frame
u = velocity[:,0]
v = velocity[:,1]
w = velocity[:,2]
udelta = u-V0                                                    	# Axial velocity variation respects to the upstream value

vorticity = np.einsum('ij,kj->ik',VORTICITY,TRANSFORMATION)

TRANS_strain = np.einsum('ijm,kj->ikm',STRAIN,TRANSFORMATION)  	# Matrix product: M.STRAIN   (M=transformation matrix)
TRANS_T = np.transpose(TRANSFORMATION)							# M transpose
strain = np.einsum('km,ijk->ijm',TRANS_T,TRANS_strain)     		# Strain tensor transformation: M.STRAIN.Mtranspose
 
tau = MU_EFF*strain

Cp_gradient = gradient(Cp)   											# Cp gradient in the mesh reference frame
Cp_gradient = np.einsum('ij,kj->ik',Cp_gradient,TRANSFORMATION)[:,2]	# Cp gradient in the aerodynamic reference frame

"""
if InputFormat==13:
	where_are_NaNs = isnan(u)    
	u[where_are_NaNs] = 0
	
	where_are_NaNs = isnan(Rho) 
	Rho[where_are_NaNs] = 0
"""

Lamb_Vector = algs.cross(velocity,vorticity)



############################################################################################################
########################################## COMMON TO ALL METHODS ###########################################
############################################################################################################


# We define first if the far field values will be adimentionalized or not:
if Nondimensionalize==1:
    AdimF = 0.5*Rho0*Sref*V0**2                                                             # Force adimensionalization factor 
    AdimP = 0.5*Rho0*Sref*V0**3                                                             # Power adimensionalization factor 
    print ""
    print "# EXERGY/LIFT/DRAG OUTPUT VALUES: nondimensionalized"
else:
    AdimF=1
    AdimP=1
    print ""
    print "# EXERGY/LIFT/DRAG OUTPUT VALUES: absolute"

	
############################################################################################################
############################################ NEAR-FIELD METHODS ############################################
############################################################################################################


if NEAR_FIELD == 1:    # If activated, this option allows the calculation of the parameters related to the Near-field method 
	TRANS_V_gradient = np.einsum('ijm,kj->ikm',V_gradient,TRANSFORMATION)  	# Matrix product: M.Vgradient   (M=transformation matrix)
	TRANS_T = np.transpose(TRANSFORMATION)									# M transpose
	V_gradient = np.einsum('km,ijk->ijm',TRANS_T,TRANS_V_gradient)     		# Vgradient tensor transformation: M.Vgradient.Mtranspose
	DimF = 0.5*Rho0*Sref*V0**2												# Cd to force factor

############################################################################################################
############################################# FAR-FIELD METHODS ############################################
############################################################################################################

	
if FAR_FIELD == 1:    # If activated, this option allows the calculation of the parameters related to the far-field method 
		
	########### MOMENTUM CONSERVATION METHOD ###########
	MOMENTUM_total_drag = (-1*Rho*u*(u-V0)-(Ps-Ps0))/AdimF                                      # Total drag density distribution 
	MOMENTUM_u = (-1*Rho*u*(u-V0))/AdimF                                                        # Speed-component of the total drag density
	MOMENTUM_p = (-1*(Ps-Ps0))/AdimF                                                            # Pressure-component of the total drag density
	MOMENTUM_lift = (-1*Rho*u*w)/AdimF                                                          # Lift density distribution
	MOMENTUM_sideforce = (-1*Rho*u*v)/AdimF                                                     # Sideforce density distribution
	MOMENTUM_stress = 2*MU_LAM*strain[:,0,0]

	
	########### ONORATO METHOD ###########
	ONORATO_Pt = (Pt0-Pt)/AdimF  
	ONORATO_u = 0.5*Rho*(u-V0)**2/AdimF  	
	ONORATO_vw = 0.5*Rho*(v**2+w**2)/AdimF	
	ONORATO_drag = ONORATO_Pt + ONORATO_u + ONORATO_vw
	

	########### BETZ ###########
	BETZ_Vartificial = algs.sqrt(u**2+2*(Pt0-Pt)*(1/Rho))                                             # Artificial speed
	BETZ_profile_drag = (Pt0-Pt+0.5*Rho*(BETZ_Vartificial-u)*(BETZ_Vartificial+u-2*V0))/AdimF   # Profile drag density distribution
	BETZ_p = (Pt0-Pt)/AdimF                                                                     # Pressure-component of the profile drag density
	BETZ_u = (0.5*Rho*(BETZ_Vartificial-u)*(BETZ_Vartificial+u-2*V0))/AdimF                     # Speed-component of the profile drag density
		
		
	########### JONES ###########
	JONES_profile_drag = (2*algs.sqrt(Pt-Ps)*(algs.sqrt(Pt0-Ps0)-algs.sqrt(Pt-Ps0)) )/AdimF     # Profile drag density distribution
		
		
	########### OSWATITSCH ########### 
	OSWATITSCH_profile_drag = (Ts0/V0)*(Rho*u*DeltaS)/AdimF # profile drag density distribution

		
	########### MASKELL ########### 
	"""
	ub = np.trapz(BE_Vart-u, y)/(2*AreaTunel)
	MASKELL_profile_drag = (Pt0-Pt+0.5*Rho*(BETZ_Vartificial-u)*(BETZ_Vartificial+u-2*(V0+ub)))/AdimF              # integrand of Betz drag equation
	RESOLVER CON UN FILTRO EXTERNO §§§§§§§
	"""
	MASKELL_vortex_drag = 0.5*Rho*(v**2+w**2)/AdimF                                             # Vortex drag density
	MASKELL_lift = (Rho0*V0*vorticity[:,0]*y + Rho0*(V0-u)*w)/AdimF                                                  # Local lift density	
	MASKELL_sideforce = (Rho0*V0*vorticity[:,0]*z + Rho0*(V0-u)*v)/AdimF                                             # Local sideforce density


	########### VAN DEN VOOREN ########### 
	Root = 1-((np.e**((Gamma-1)*DeltaS/(Gamma*R))-1)/(0.5*(Gamma-1)*M0**2))+(2*(Ht-Ht0)/V0**2)  # Term inside the root of DeltaU
	VDV_DeltaUbar = V0*(algs.sqrt(Root)-1)                                                         # velocity deficit related to viscous effect (it ignores the potential effect)
	VDV_DeltaUstar = udelta-VDV_DeltaUbar                                                          # velocity defect related to potential effects
	VDV_vortex_drag = -1*(Rho*u*(u-V0-VDV_DeltaUbar)+Ps-Ps0)/AdimF                                 # Vortex drag density
	VDV_profile_drag = -1*(Rho*u*VDV_DeltaUbar)/AdimF                                              # Profile drag density
	VDV_total_drag = VDV_profile_drag + VDV_vortex_drag                                         # Total drag density


	########### GILES ###########
	GILES_profile_drag = ((Ps0*DeltaS/R)-(Rho0*DeltaHt))/AdimF                                    # Profile drag density


	########### KUSUNOSE ###########
	KUSUNOSE_vw = (0.5*Rho*(v**2+w**2))/AdimF                                                       # Traverse flow drag density (It Does not belongs to the classical momentum drag) 
	KUSUNOSE_u2 = (-0.5*Rho*(u-V0)**2)/AdimF                                                        # Coupled drag density (It Does not belongs to the classical momentum drag) 
	KUSUNOSE_vw_u2 = KUSUNOSE_vw+KUSUNOSE_u2                                                                # Traverse+coupled drag density


	KUSUNOSE_profile_drag = ((Ps0*DeltaS/R)-(Rho0*DeltaHt)-0.5*Ps0*(DeltaS/R)**2+Rho0*DeltaHt*((DeltaS/R)-0.5*DeltaHt*(M0/V0)**2))/AdimF      # Profile drag density
	KUSUNOSE_vortex_drag = 0.5*Rho0*((v**2+w**2)-(1-M0**2)*(u-V0)**2)/AdimF                                                                   # Vortex drag density
	KUSUNOSE_lift = ( Rho0*V0*vorticity[:,0]*y - Rho0*V0**2*(1-M0**2)*(w/V0**2)*(u-V0) + M0**2 *(Gamma*Ps0*w*DeltaS/(R*V0)) - Rho0*M0**2*DeltaHt*(w/V0) )/AdimF      # Local lift density                                                  # Local lift density

	"""
	# This is the formulation provided by Meheut in his PhD report, retained here for debug purposes:
	KUSUNOSE_profile_drag =(2/Sref)*(((DeltaS)/(R*Gamma*M0**2)) - ((Ht-Ht0)/V0**2) - ((DeltaS**2)/(2*R**2*Gamma*M0**2)) + ((Ht-Ht0)/V0**2)*((DeltaS/R)-0.5*M0**2*((Ht-Ht0)/V0**2)))
	KUSUNOSE_profile_drag = KUSUNOSE_profile_drag*0.5*Rho0*Sref*V0**2/AdimF                             # Profile drag density
	KUSUNOSE_vortex_drag = (0.5*Rho0*(v**2+w**2)) - 0.5*Rho0*(1-M0**2)*(VDV_DeltaUbar)**2
	KUSUNOSE_vortex_drag = KUSU_vortex_drag/AdimF                                                   # Vortex drag density
	"""
		
	KUSUNOSE_total_drag = KUSUNOSE_profile_drag + KUSUNOSE_vortex_drag                                      # Total drag density (note that its values are alredy affected by the threshold)

			
	########### MEHEUT ###########
	Raiz = 1- (((Ps/Ps0)**((Gamma-1)/Gamma)-1)/(0.5*(Gamma-1)*M0**2)) - ((v**2+w**2)/V0**2)            # Term inside the root of DeltaUstar
	DeltaUstar = algs.sqrt(Raiz)-1                                                              # Deficit of velocity related to the potential effect
	DeltaUonera = (u/V0)-1                                                                      # Total deficit of velocity 
	DeltaUline = DeltaUonera-DeltaUstar                                                         # Deficit of velocity related to the viscous effects
	DeltaPt = (Pt/Pt0)-1                                                                        # Total pressure drop ratio
	DeltaTt = (Tt/Tt0)-1                                                                        # Total temperature drop ratio
	MEHEUT_profile_drag = (1/Sref)* ((-2*DeltaPt/(Gamma*M0**2)) - DeltaTt - (1-0.25*M0**2)*DeltaTt**2 - DeltaPt*DeltaTt - (1-M0**2)*(DeltaUline**2+2*DeltaUline*DeltaUstar))
	MEHEUT_profile_drag = MEHEUT_profile_drag*0.5*Rho0*Sref*V0**2/AdimF                         # Profile drag density



############################################################################################################
############################################ LAMB VECTOR METHODS ###########################################
############################################################################################################


if LAMB_VECTOR == 1:    # If activated, this option allows the calculation of the parameters related to the Lamb vector method 
	
	r = points/(Dimension-1)

	
	########### WU ###########
	WU_lift = Rho*Lamb_Vector[:,2]/AdimF 
	WU_induced_drag = Rho*Lamb_Vector[:,0]/AdimF
	Plane_normal = np.array([[1,0,0]])   								# Plane normal in the aerodynamic reference frame
	n = np.repeat(Plane_normal,algs.shape(Lamb_Vector)[0],axis=0)
	WU_profile_drag = Rho*algs.cross(r,algs.cross(n,Lamb_Vector))/AdimF

	
	########### MELE ###########
	RHO_GRADIENT = gradient(Rho)
	V2_GRADIENT = gradient(0.5*V**2)
	rho_gradient = np.einsum('ij,kj->ik',RHO_GRADIENT,TRANSFORMATION)
	v2_gradient = np.einsum('ij,kj->ik',V2_GRADIENT,TRANSFORMATION)
	mrho = algs.cross(r,algs.cross(rho_gradient,v2_gradient))
	
	
	MELE_lift = (Rho*Lamb_Vector[:,2] - mrho[:,2])/AdimF 
	MELE_induced_drag = ( Rho*Lamb_Vector[:,0] - mrho[:,0])/AdimF
	MELE_profile_drag = Rho*algs.cross(r,algs.cross(n,Lamb_Vector))/AdimF
	
	
	
############################################################################################################
############################################## EXERGY METHODS ##############################################
############################################################################################################

if EXERGY == 1:    # If activated, this option allows the calculation of the parameters related to the Exergy method 

	#############################
	########### ARNTZ ###########
	#############################
	ARNTZ_Eth_t = ((Rho*CvGas*Ts*u*(1-Ts0*algs.log(Ts/Ts0)/Ts))-(Rho*CvGas*Ts0*u))/AdimP       			# Thermal exergy T-component (constant volume) - exact formulation -
	ARNTZ_Eth_p = ((Ps0*(1-(Rho/Rho0)*algs.log(Rho0/Rho))*u)-(Rho*R*Ts0*u))/AdimP             			# Thermal exergy P-component (variable volume) - exact formulation -
	#ARNTZ_Eth_t = (0.5*Rho*CvGas*(Ts-Ts0)**2*u/Ts0)/AdimP                                              # Thermal exergy T-component (constant volume) - approximative formulation -
	#ARNTZ_Eth_p = ((Rho*R*Ts0*((Rho0/Rho)+algs.log(Rho/Rho0))*u)-(Rho*R*Ts0*u))/AdimP                 	# Thermal exergy P-component (variable volume) - alternative formulation -
	ARNTZ_A = Ts0*Rho*DeltaS*u/AdimP                							    					# Exact value of the anergy created 
	ARNTZ_Aphi = (Ts0/Ts)*Phi_eff/AdimP                                             					# Viscous anergy
	ARNTZ_At = (Ts0*Keff*algs.mag(Ts_gradient)**2)/(AdimP*Ts**2)                              			# Thermal anergy
	ARNTZ_Ep = ((Ps-Ps0)*(u-V0))/AdimP                                              					# Pressure exergy
	ARNTZ_Eu = (0.5*Rho*(u-V0)**2*u)/AdimP                                          					# Axial speed exergy
	ARNTZ_Ev = (0.5*Rho*(v**2+w**2)*u)/AdimP                                        					# Transverse speed exergy
	ARNTZ_Em = ARNTZ_Ep + ARNTZ_Eu + ARNTZ_Ev                                     						# Total mechanical exergy 
	ARNTZ_Eth = ARNTZ_Eth_t + ARNTZ_Eth_p                       										# Thermal exergy 
	ARNTZ_Cd = ARNTZ_A + ARNTZ_Em + ARNTZ_Eth  															# Exergy based drag coefficient
	
	ARNTZ_Aw_div = algs.divergence(Ts0*Rho*DeltaS*Vector_Speed)/AdimP
	MASK = Shock_Detector > 0.9  																		# This detects the shockwave regions
	ARNTZ_Aw_div_MASKED = 1*MASK*ARNTZ_Aw_div
	

	###############################
	########### AGUIRRE ###########
	###############################

	# Wave anergy with masking method:
	MASK = Q_criterion < 0.0  																# This detects the viscous regions
	AGUIRRE_Aw_div_MASKED = 1*MASK*algs.divergence(Ts0*Rho*DeltaS*Vector_Speed)/AdimP		# This mask the wave anergy in the viscous regions

	
	# Velocity decomposition method
	ETA = 1+0.5*(Gamma-1)*M0**2*(1-((u**2+v**2+w**2)*Tt0/(V0**2*Tt)))
	RootVabsstar = 1- ((ETA*(Pt/Pt0)**((Gamma-1)/Gamma)-1)/(0.5*(Gamma-1)*M0**2))            # Term inside the root of Vabsstar
	AGUIRRE_Vabs_star = V0*algs.sqrt(RootVabsstar)

	FlowTita = algs.arctan(w/algs.sqrt(u**2+v**2))
	FlowAlfa = algs.arctan(w/u)
	FlowBeta = algs.arctan(v/u)

	AGUIRRE_u_star = AGUIRRE_Vabs_star*algs.cos(FlowTita)*algs.cos(FlowBeta)
	AGUIRRE_v_star = AGUIRRE_Vabs_star*algs.cos(FlowTita)*algs.sin(FlowBeta)
	AGUIRRE_w_star = AGUIRRE_Vabs_star*algs.sin(FlowTita)

	AGUIRRE_u_bar = u-AGUIRRE_u_star
	AGUIRRE_v_bar = v-AGUIRRE_v_star
	AGUIRRE_w_bar = w-AGUIRRE_w_star

	# AERO parameters decomposition
	AGUIRRE_Ts_star = Ts0*(1+0.5*(Gamma-1)*M0**2*(1-(AGUIRRE_Vabs_star/V0)**2))
	AGUIRRE_Ts_bar = Ts-AGUIRRE_Ts_star
	AGUIRRE_Rho_star = Rho0*(1+0.5*(Gamma-1)*M0**2*(1-(AGUIRRE_Vabs_star/V0)**2))**(1/(Gamma-1))
	AGUIRRE_Rho_bar = Rho-AGUIRRE_Rho_star
	AGUIRRE_e_star = CvGas*AGUIRRE_Ts_star
	AGUIRRE_e_bar = eint-AGUIRRE_e_star
	AGUIRRE_Hs_star = CpGas*AGUIRRE_Ts_star
	AGUIRRE_Hs_bar = Hs-AGUIRRE_Hs_star

	
	"""
	# OLD FORMULATION: this uses the definition of Em, Ep and Eth proposed by Arntz (it is kept here for testing). For more info see paper AEC2020 "A BETTER ASSESSMENT OF THE RECOVERABLE ENERGY BEHIND A BODY BY THE EXERGY METHOD"
	# Exergy decomposition
	AGUIRRE_Eu_star = (0.5*Rho*(AGUIRRE_u_star-V0)**2*AGUIRRE_u_star)/AdimP
	AGUIRRE_Eu_bar = ARNTZ_Eu - AGUIRRE_Eu_star
	AGUIRRE_Ev_star = (0.5*Rho*(AGUIRRE_v_star**2+AGUIRRE_w_star**2)*AGUIRRE_u_star)/AdimP
	AGUIRRE_Ev_bar = ARNTZ_Ev - AGUIRRE_Ev_star
	AGUIRRE_Ep_star = ((Ps-Ps0)*(AGUIRRE_u_star-V0))/AdimP 
	AGUIRRE_Ep_bar = ARNTZ_Ep - AGUIRRE_Ep_star
	AGUIRRE_Em_star = AGUIRRE_Eu_star + AGUIRRE_Ev_star + AGUIRRE_Ep_star
	AGUIRRE_Em_bar = AGUIRRE_Eu_bar + AGUIRRE_Ev_bar + AGUIRRE_Ep_bar 
	AGUIRRE_Eth_t_star = ((AGUIRRE_Rho_star*CvGas*AGUIRRE_Ts_star*AGUIRRE_u_star*(1-Ts0*algs.log(AGUIRRE_Ts_star/Ts0)/AGUIRRE_Ts_star))-(AGUIRRE_Rho_star*CvGas*Ts0*AGUIRRE_u_star))/AdimP       			
	AGUIRRE_Eth_p_star = ((Ps0*(1-(AGUIRRE_Rho_star/Rho0)*algs.log(Rho0/AGUIRRE_Rho_star))*AGUIRRE_u_star)-(AGUIRRE_Rho_star*R*Ts0*AGUIRRE_u_star))/AdimP             			
	AGUIRRE_Eth_star = AGUIRRE_Eth_t_star + AGUIRRE_Eth_p_star 
	AGUIRRE_Eth_bar = ARNTZ_Eth-AGUIRRE_Eth_star
	AGUIRRE_Cd_star = AGUIRRE_Em_star + AGUIRRE_Eth_star 
	AGUIRRE_Cd_bar = ARNTZ_A + AGUIRRE_Em_bar + AGUIRRE_Eth_bar 
	"""
	
	# NEW FORMULATION: this uses the new definition of Em, Ep and Eth (see paper AEC2020 "A BETTER ASSESSMENT OF THE RECOVERABLE ENERGY BEHIND A BODY BY THE EXERGY METHOD")
	# (Unlike the others Aguirre equations, these particular formulations require an infinite surface integral)
	
	AGUIRRE_INF_Ep = ARNTZ_Ep + ARNTZ_Eth_p  								# Redefined Ep
	AGUIRRE_INF_Em = ARNTZ_Eu + ARNTZ_Ev + AGUIRRE_INF_Ep 				  	# Redefined Em
	AGUIRRE_INF_Eth = ARNTZ_Eth_t                     				   		# Redefined Eth
	AGUIRRE_INF_Cd = ARNTZ_A + AGUIRRE_INF_Em + AGUIRRE_INF_Eth  			# Exergy based drag coefficient

	
	# Exergy decomposition
	AGUIRRE_Eth_t_star = ((AGUIRRE_Rho_star*CvGas*AGUIRRE_Ts_star*AGUIRRE_u_star*(1-Ts0*algs.log(AGUIRRE_Ts_star/Ts0)/AGUIRRE_Ts_star))-(AGUIRRE_Rho_star*CvGas*Ts0*AGUIRRE_u_star))/AdimP       			
	AGUIRRE_Eth_p_star = ((Ps0*(1-(AGUIRRE_Rho_star/Rho0)*algs.log(Rho0/AGUIRRE_Rho_star))*AGUIRRE_u_star)-(AGUIRRE_Rho_star*R*Ts0*AGUIRRE_u_star))/AdimP             			
	AGUIRRE_Eu_star = (0.5*Rho*(AGUIRRE_u_star-V0)**2*AGUIRRE_u_star)/AdimP
	AGUIRRE_Eu_bar = ARNTZ_Eu - AGUIRRE_Eu_star
	AGUIRRE_Ev_star = (0.5*Rho*(AGUIRRE_v_star**2+AGUIRRE_w_star**2)*AGUIRRE_u_star)/AdimP
	AGUIRRE_Ev_bar = ARNTZ_Ev - AGUIRRE_Ev_star
	AGUIRRE_Ep_star_original = ((Ps-Ps0)*(AGUIRRE_u_star-V0))/AdimP 
	AGUIRRE_Ep_star = AGUIRRE_Ep_star_original + AGUIRRE_Eth_p_star
	#AGUIRRE_Ep_bar = ARNTZ_Ep - AGUIRRE_Ep_star ###############borrar
	AGUIRRE_Ep_bar = AGUIRRE_INF_Ep - AGUIRRE_Ep_star
	AGUIRRE_Em_star = AGUIRRE_Eu_star + AGUIRRE_Ev_star + AGUIRRE_Ep_star
	AGUIRRE_Em_bar = AGUIRRE_Eu_bar + AGUIRRE_Ev_bar + AGUIRRE_Ep_bar 
	AGUIRRE_Eth_star = AGUIRRE_Eth_t_star  
	#AGUIRRE_Eth_bar = ARNTZ_Eth - AGUIRRE_Eth_star    #borrarrrrrrrrrrrrrrrrr
	AGUIRRE_Eth_bar = AGUIRRE_INF_Eth - AGUIRRE_Eth_star
	AGUIRRE_Cd_star = AGUIRRE_Em_star + AGUIRRE_Eth_star 
	AGUIRRE_Cd_bar = ARNTZ_A + AGUIRRE_Em_bar + AGUIRRE_Eth_bar 

	#Correction method
	AGUIRRE_Cd_bar_corrected = AGUIRRE_Cd_bar + algs.abs(AGUIRRE_Cd_star)
	AGUIRRE_Em_bar_corrected = AGUIRRE_Em_bar + algs.abs(AGUIRRE_Em_star)       #Expression valid ONLY in incompressible flow!!!
	
	

	  



############################################################################################################
###############################################  PRINT INFO ################################################
############################################################################################################

# This info is useful for the user in order to create the slice plane normal to the upstream direction
print ""
print "# NORMAL DIRECTION TO USE FOR SLICE PLANE OR INTERSECTION LINE DEFINITION"

if UpwardAxis==0:    #Mesh Y-axis pointing upwards
	Plane_NORMAL_X = np.cos(Alfa*np.pi/180)*np.cos(Beta*np.pi/180)
	Plane_NORMAL_Y = np.sin(Alfa*np.pi/180)
	Plane_NORMAL_Z = -1*np.sin(Beta*np.pi/180)
	print "Normal= ",Plane_NORMAL_X, "  ",Plane_NORMAL_Y, "  ", Plane_NORMAL_Z
	
elif UpwardAxis==1:    #Mesh Z-axis pointing upwards
	Plane_NORMAL_X = np.cos(Alfa*np.pi/180)*np.cos(Beta*np.pi/180)
	Plane_NORMAL_Y = np.sin(Beta*np.pi/180)
	Plane_NORMAL_Z = np.sin(Alfa*np.pi/180)
	print "Normal= ",Plane_NORMAL_X, "  ",Plane_NORMAL_Y, "  ", Plane_NORMAL_Z

#output.FieldData.append(Plane_NORMAL_X, "Plane_NORMAL_X")
#output.FieldData.append(Plane_NORMAL_Y, "Plane_NORMAL_Y")
#output.FieldData.append(Plane_NORMAL_Z, "Plane_NORMAL_Z")

############################################################################################################
###############################################  DATA OUTPUT ###############################################
############################################################################################################


if AERO == 1:    # If activated, this option sends to the output the arrays related to the aerodynamic parameters (allows memory allocation) 

	# Constant values:
	output.FieldData.append(R, "R")
	output.FieldData.append(Gamma, "Gamma")
	output.FieldData.append(CpGas, "CpGas")
	output.FieldData.append(CvGas, "CvGas")
	output.FieldData.append(Alfa, "Alfa")
	output.FieldData.append(Beta, "Beta")
	output.FieldData.append(M0, "M0")
	output.FieldData.append(V0, "V0")
	output.FieldData.append(Pt0, "Pt0")
	output.FieldData.append(Ps0, "Ps0")
	output.FieldData.append(Tt0, "Tt0")
	output.FieldData.append(Ts0, "Ts0")
	output.FieldData.append(Rho0, "Rho0")
	output.FieldData.append(AdimP, "AdimP")
	output.FieldData.append(AdimF, "AdimF")
	output.FieldData.append(UpwardAxis, "UpwardAxis")
	output.FieldData.append(TRANSFORMATION, "TRANSFORMATION")



	# Basic aero data:
	output.PointData.append(inputs[0].PointData["Points"], "Points")
	output.PointData.append(MU_LAM, "AERO_MU_LAM")
	output.PointData.append(MU_TURB, "AERO_MU_TURB")
	output.PointData.append(MU_EFF, "AERO_MU_EFF")
	output.PointData.append(Keff, "AERO_Keff")
	output.PointData.append(Vector_Speed, "AERO_Vector_Speed")
	output.PointData.append(u, "AERO_u")
	output.PointData.append(v, "AERO_v")
	output.PointData.append(w, "AERO_w")
	output.PointData.append(udelta, "AERO_udelta")
	output.PointData.append(V, "AERO_Vabs")
	output.PointData.append(M, "AERO_M")
	output.PointData.append(vorticity, "AERO_Vorticity")
	output.PointData.append(Helicity, "AERO_Helicity")
	output.PointData.append(tau, "AERO_Stress(tau)")
	output.PointData.append(strain, "AERO_Strain")
	output.PointData.append(Q_criterion, "AERO_Q_criterion")
	output.PointData.append(Lambda_2, "AERO_Lambda_2")
	output.PointData.append(Phi_eff, "AERO_Phi_eff")
	output.PointData.append(Ps, "AERO_Ps")
	output.PointData.append(Pt, "AERO_Pt")
	output.PointData.append(PtPt0, "AERO_Pt/Pt0")
	output.PointData.append(Shock_Detector, "AERO_Shock_Detector")
	output.PointData.append(q, "AERO_q")
	output.PointData.append(Cp, "AERO_Cp")
	
	
	output.PointData.append(Cp_gradient, "AERO_Cp_gradient")
	
	
	output.PointData.append(Ts, "AERO_Ts")
	output.PointData.append(Tt, "AERO_Tt")
	output.PointData.append(Hs, "AERO_Hs")
	output.PointData.append(Ht, "AERO_Ht")
	output.PointData.append(DeltaHt, "AERO_Ht-Ht0")
	output.PointData.append(Rho, "AERO_Rho")
	output.PointData.append(DeltaS, "AERO_s-s0")
	output.PointData.append(eint, "AERO_e")

	

if NEAR_FIELD == 1:    # If activated, this option sends to the output the arrays related to the near-field method (allows memory allocation) 
	output.PointData.append(V_gradient, "AERO_V_gradient")
	output.FieldData.append(Sref, "Sref")
	output.FieldData.append(DimF, "DimF")
	
	

if FAR_FIELD == 1:    # If activated, this option sends to the output the arrays related to the far-field method (allows memory allocation) 
	# Momentum conservation:
	output.PointData.append(MOMENTUM_total_drag, "MOMENTUM_total_drag")
	output.PointData.append(MOMENTUM_u, "MOMENTUM_u")
	output.PointData.append(MOMENTUM_p, "MOMENTUM_p")
	output.PointData.append(MOMENTUM_lift, "MOMENTUM_lift")
	output.PointData.append(MOMENTUM_sideforce, "MOMENTUM_sideforce")
	output.PointData.append(MOMENTUM_stress, "MOMENTUM_stress")
	
	
	# Onorato:
	output.PointData.append(ONORATO_drag, "ONORATO_drag")
	output.PointData.append(ONORATO_Pt, "ONORATO_Pt")
	output.PointData.append(ONORATO_u, "ONORATO_u")
	output.PointData.append(ONORATO_vw, "ONORATO_vw")

	# Betz:
	output.PointData.append(BETZ_Vartificial, "BETZ_Vartificial")
	output.PointData.append(BETZ_profile_drag, "BETZ_profile_drag")
	output.PointData.append(BETZ_p, "BETZ_p")
	output.PointData.append(BETZ_u, "BETZ_u")

	# Jones:
	output.PointData.append(JONES_profile_drag, "JONES_profile_drag")

	# Oswatitsch:
	output.PointData.append(OSWATITSCH_profile_drag, "OSWATITSCH_profile_drag")

	# Maskell:
	output.PointData.append(MASKELL_vortex_drag, "MASKELL_vortex_drag")
	output.PointData.append(MASKELL_lift, "MASKELL_lift")
	output.PointData.append(MASKELL_sideforce, "MASKELL_sideforce")

	# Van Der Vooren:
	output.PointData.append(VDV_DeltaUbar, "VDV_DeltaUbar")
	output.PointData.append(VDV_DeltaUstar, "VDV_DeltaUstar")
	output.PointData.append(VDV_vortex_drag, "VDV_vortex_drag")
	output.PointData.append(VDV_profile_drag, "VDV_profile_drag")
	output.PointData.append(VDV_total_drag, "VDV_total_drag")

	# Giles:
	output.PointData.append(GILES_profile_drag, "GILES_profile_drag")

	# Kusunose:
	#output.PointData.append(KUSUNOSE_vw, "KUSUNOSE_vw")
	#output.PointData.append(KUSUNOSE_u2, "KUSUNOSE_u2")
	#output.PointData.append(KUSUNOSE_vw_u2, "KUSUNOSE_vw_u2")
	output.PointData.append(KUSUNOSE_vortex_drag, "KUSUNOSE_vortex_drag")
	output.PointData.append(KUSUNOSE_profile_drag, "KUSUNOSE_profile_drag")
	output.PointData.append(KUSUNOSE_total_drag, "KUSUNOSE_total_drag")
	output.PointData.append(KUSUNOSE_lift, "KUSUNOSE_lift")

	# Meheut:
	output.PointData.append(MEHEUT_profile_drag, "MEHEUT_profile_drag")
	
	
	if AERO == 0:  
		output.PointData.append(vorticity, "AERO_Vorticity")   #This variable is used later by Epsilon_Profile_Drag_Breakdown 


if LAMB_VECTOR == 1:    # If activated, this option sends to the output the arrays related to the Lamb Vector method (allows memory allocation) 
	
	output.PointData.append(Lamb_Vector, "Lamb_Vector")
	output.PointData.append(MELE_lift, "MELE_lift")
	output.PointData.append(MELE_induced_drag, "MELE_induced_drag")
	output.PointData.append(MELE_profile_drag, "MELE_profile_drag")
	output.PointData.append(WU_lift, "WU_lift")
	output.PointData.append(WU_induced_drag, "WU_induced_drag")
	output.PointData.append(WU_profile_drag, "WU_profile_drag")
	


if EXERGY == 1:    # If activated, this option sends to the output the arrays related to the Exergy method (allows memory allocation) 

	# Arntz:
	output.PointData.append(ARNTZ_Ep, "ARNTZ_Ep")
	output.PointData.append(ARNTZ_Eu, "ARNTZ_Eu")
	output.PointData.append(ARNTZ_Ev, "ARNTZ_Ev")
	output.PointData.append(ARNTZ_Em, "ARNTZ_Em")
	output.PointData.append(ARNTZ_Eth, "ARNTZ_Eth")
	output.PointData.append(ARNTZ_Eth_p, "ARNTZ_Eth_p")
	output.PointData.append(ARNTZ_Eth_t, "ARNTZ_Eth_t")
	output.PointData.append(ARNTZ_Aphi, "ARNTZ_Aphi")
	output.PointData.append(ARNTZ_At, "ARNTZ_At")
	output.PointData.append(ARNTZ_Aw_div, "ARNTZ_Aw_div")
	output.PointData.append(ARNTZ_Aw_div_MASKED, "ARNTZ_Aw_div_MASKED")
	output.PointData.append(ARNTZ_A, "ARNTZ_A")
	output.PointData.append(ARNTZ_Cd, "ARNTZ_Cd")


	# Aguirre:
	output.PointData.append(AGUIRRE_u_star, "AGUIRRE_AERO_u_star")
	output.PointData.append(AGUIRRE_u_bar, "AGUIRRE_AERO_u_bar")
	output.PointData.append(AGUIRRE_v_star, "AGUIRRE_AERO_v_star")
	output.PointData.append(AGUIRRE_v_bar, "AGUIRRE_AERO_v_bar")
	output.PointData.append(AGUIRRE_w_star, "AGUIRRE_AERO_w_star")
	output.PointData.append(AGUIRRE_w_bar, "AGUIRRE_AERO_w_bar")
	output.PointData.append(AGUIRRE_Vabs_star, "AGUIRRE_AERO_Vabs_star")
	output.PointData.append(AGUIRRE_Ts_star, "AGUIRRE_AERO_Ts_star")
	output.PointData.append(AGUIRRE_Ts_bar, "AGUIRRE_AERO_Ts_bar")
	output.PointData.append(AGUIRRE_Rho_star, "AGUIRRE_AERO_Rho_star")
	output.PointData.append(AGUIRRE_Rho_bar, "AGUIRRE_AERO_Rho_bar")
	output.PointData.append(AGUIRRE_e_star, "AGUIRRE_AERO_e_star")
	output.PointData.append(AGUIRRE_e_bar, "AGUIRRE_AERO_e_bar")
	output.PointData.append(AGUIRRE_Hs_star, "AGUIRRE_AERO_Hs_star")
	output.PointData.append(AGUIRRE_Hs_bar, "AGUIRRE_AERO_Hs_bar")

	output.PointData.append(AGUIRRE_Em_star, "AGUIRRE_Em_star")
	output.PointData.append(AGUIRRE_Em_bar, "AGUIRRE_Em_bar")
	output.PointData.append(AGUIRRE_Eu_star, "AGUIRRE_Eu_star")
	output.PointData.append(AGUIRRE_Eu_bar, "AGUIRRE_Eu_bar")
	output.PointData.append(AGUIRRE_Ev_star, "AGUIRRE_Ev_star")
	output.PointData.append(AGUIRRE_Ev_bar, "AGUIRRE_Ev_bar")
	output.PointData.append(AGUIRRE_Ep_star, "AGUIRRE_Ep_star")
	output.PointData.append(AGUIRRE_Ep_bar, "AGUIRRE_Ep_bar")
	output.PointData.append(AGUIRRE_Eth_star, "AGUIRRE_Eth_star")
	output.PointData.append(AGUIRRE_Eth_bar, "AGUIRRE_Eth_bar")
	output.PointData.append(AGUIRRE_Cd_star, "AGUIRRE_Cd_star")
	output.PointData.append(AGUIRRE_Cd_bar, "AGUIRRE_Cd_bar")

	output.PointData.append(AGUIRRE_Em_bar_corrected, "AGUIRRE_Em_bar_corrected")
	output.PointData.append(AGUIRRE_Cd_bar_corrected, "AGUIRRE_Cd_bar_corrected")
	output.PointData.append(AGUIRRE_Aw_div_MASKED, "AGUIRRE_Aw_div_MASKED")
	
	
	output.PointData.append(AGUIRRE_INF_Ep, "AGUIRRE_INF_Ep")
	output.PointData.append(AGUIRRE_INF_Em, "AGUIRRE_INF_Em")
	output.PointData.append(AGUIRRE_INF_Eth, "AGUIRRE_INF_Eth")
	output.PointData.append(AGUIRRE_INF_Cd, "AGUIRRE_INF_Cd")
	



############################################################################################################
###############################################  END OF CODE ###############################################
############################################################################################################
