# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################              Integration Tool Plugin             #############################
############################################################################################################

# Abstract:
This Paraview's programmable filter takes the output from Epsilon_Aero_Data and integrates its variables
depending upon the type of variable (survey plane integral or volume integral). Details of the formulations
and how to use the plugin are available in the User's Manual.


# Disclaimer:
This plugin was developed for research purposes and released under GNU GPL V3 license without any guarantee
of any kind, either express or implied. Its use for teaching, research and industrial applications is allowed
under the strict responsibility of the user. The authors of this plugin are not responsible for any damages
or errors related to the use of the plugin.
"""


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import vtk
from vtk.numpy_interface import algorithms as algs
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np
import vtk.vtkFiltersParallel


############################################################################################################
############################################# INPUT PARAMETERS #############################################
############################################################################################################

# Input parameters are defined as string in Paraview's GUI, thus a string to float convertion is required:
Output_Type = float(Output_Type)				# 0=Line distribution  1=3D to 2D mapping
axis = float(axis)								# 0=Aerodynamic x-axis  1=Aerodynamic y-axis  2=Aerodynamic z-axis  3=Mesh X-axis  4=Mesh Y-axis  5=Mesh Z-axis
Select_Action = float(Select_Action)			# 0=Visualize Line / Mapping Plane  1=Visualize Intersection Plane  2=Perform Automatic Integration
Select_Variable = float(Select_Variable)   		# 0=Infinite Surface Formulations  1=Wake Surface Formulations  2=Volume Formulations  3=Body Surface Formulations  4=Profile Drag Breakdown  5=Maskell lift distribution 6=AERO parameters
FlipClipNormal = float(FlipClipNormal)

numPts = NUMPTS[0]								# Number of points along the first distribution line
Start = START[0]								# Star line coordinate [m]
End = END[0]									# End line coordinate [m]

numPts1 = NUMPTS[1]								# Number of points along the second distribution line
Start1 = START[1]								# Star line coordinate [m]
End1 = END[1]									# End line coordinate [m]

############################################################################################################
############################################## READ INPUT DATA #############################################
############################################################################################################

TRANSFORMATION = inputs[0].FieldData["TRANSFORMATION"]
T_t = np.transpose(TRANSFORMATION)
T_inverse = np.dot(T_t,np.dot(TRANSFORMATION,T_t))  	# It calculates the inverse of the TRANSFORMATION matrix


############################################################################################################
#################################  REFERENCE LINE FOR SURVEY PLANE POSITION ################################
############################################################################################################

# Define linepoints and plane normal:
if Output_Type == 0:  											# Line distribution selected by the user

	if axis == 0:												# Aero x-axis selected by user
		X = np.linspace(Start,End,numPts) 						# Creates a uniform distribution of points along the mesh X-axis
		Y = np.linspace(0,0,numPts)								# Creates an array filled with zeros
		Z = np.linspace(0,0,numPts)
		POINTS = algs.make_vector(X,Y,Z) 						# Creates a vector with the point coordinates
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)  		# Transforms the points from the mesh reference frame to the aero reference frame
		normal = np.array([1,0,0])								# Define the normal of the survey plane on the mesh reference frame
		NORMAL = np.dot(T_inverse,normal)   					# Express the normal in the aero reference frame
			
		
	if axis == 1:
		X = np.linspace(0,0,numPts)
		Y = np.linspace(Start,End,numPts)
		Z = np.linspace(0,0,numPts)
		POINTS = algs.make_vector(X,Y,Z) 
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)
		normal = np.array([0,1,0])
		NORMAL = np.dot(T_inverse,normal)

	if axis == 2:
		X = np.linspace(0,0,numPts)
		Y = np.linspace(0,0,numPts)
		Z = np.linspace(Start,End,numPts)
		POINTS = algs.make_vector(X,Y,Z) 
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)
		normal = np.array([0,0,1])
		NORMAL = np.dot(T_inverse,normal)

	if axis == 3:
		X = np.linspace(Start,End,numPts)
		Y = np.linspace(0,0,numPts)
		Z = np.linspace(0,0,numPts)
		POINTS = algs.make_vector(X,Y,Z) 
		normal = np.array([1,0,0])
		NORMAL = normal
		
	if axis == 4:
		X = np.linspace(0,0,numPts)
		Y = np.linspace(Start,End,numPts)
		Z = np.linspace(0,0,numPts)
		POINTS = algs.make_vector(X,Y,Z)
		normal = np.array([0,1,0])
		NORMAL = normal

	if axis == 5:
		X = np.linspace(0,0,numPts)
		Y = np.linspace(0,0,numPts)
		Z = np.linspace(Start,End,numPts)
		POINTS = algs.make_vector(X,Y,Z) 
		normal = np.array([0,0,1])
		NORMAL = normal

	
if FlipClipNormal == 1:
	NORMAL = -1*NORMAL
		
		
############################################################################################################
###############################  REFERENCE MESH FOR MAPPING PLANE DEFINITION ###############################
############################################################################################################
		
# Define the points for the mesh used in the 3D to 2D mapping:
if  Output_Type == 1:   																					# 3D to 2D mapping selected by user
 
 	if axis == 0:    																						# Aero x-axis selected
		Y = np.linspace(Start,End,numPts)  																	# Creates points along mesh Y-axis
		Z = np.linspace(Start1,End1,numPts1)																# Creates points along mesh Z-axis
		Ymesh, Zmesh = np.meshgrid(Y, Z)  																	# Creates the points of the mesh
		Xmesh = 0*Ymesh  																					# Creates an array with zeros
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F')) 	# Ravel converts the matrix-like arrays into vector-like arrays
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)													# The mesh points are converted from MESH reference frame to aero frame
		normal = np.array([0,1,0])																			# Survey plane normal in mesh reference frame
		NORMAL = np.dot(T_inverse,normal)   																# Survey plane normal in aero reference frame
		NORMAL_int = np.array([0,0,1])																		# Intersection plane normal in mesh reference frame
		NORMAL_int = np.dot(T_inverse,NORMAL_int)															# Intersection plane normal in aero reference frame
		
 	if axis == 1:
		X = np.linspace(Start,End,numPts)
		Z = np.linspace(Start1,End1,numPts1)		
		Xmesh, Zmesh = np.meshgrid(X, Z)
		Ymesh = 0*Xmesh
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F'))
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)
		normal = np.array([1,0,0])
		NORMAL = np.dot(T_inverse,normal)   			# Survey plane normal
		NORMAL_int = np.array([0,0,1])		
		NORMAL_int = np.dot(T_inverse,NORMAL_int)		# Intersection plane normal

 	if axis == 2:
		X = np.linspace(Start,End,numPts)
		Y = np.linspace(Start1,End1,numPts1)		
		Xmesh, Ymesh = np.meshgrid(X, Y)
		Zmesh = 0*Xmesh
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F'))
		POINTS = np.einsum('ij,kj->ik',POINTS,T_inverse)
		normal = np.array([1,0,0])
		NORMAL = np.dot(T_inverse,normal)   			# Survey plane normal
		NORMAL_int = np.array([0,1,0])		
		NORMAL_int = np.dot(T_inverse,NORMAL_int)		# Intersection plane normal
		
 	if axis == 3:
		Y = np.linspace(Start,End,numPts)
		Z = np.linspace(Start1,End1,numPts1)		
		Ymesh, Zmesh = np.meshgrid(Y, Z)
		Xmesh = 0*Ymesh
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F'))
		normal = np.array([0,1,0])
		NORMAL = np.dot(T_inverse,normal)   			# Survey plane normal
		NORMAL_int = np.array([0,0,1])		
		NORMAL_int = np.dot(T_inverse,NORMAL_int)		# Intersection plane normal
	
 	if axis == 4:
		X = np.linspace(Start,End,numPts)
		Z = np.linspace(Start1,End1,numPts1)		
		Xmesh, Zmesh = np.meshgrid(X, Z)
		Ymesh = 0*Xmesh
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F'))
		normal = np.array([1,0,0])
		NORMAL = np.dot(T_inverse,normal)   			# Survey plane normal
		NORMAL_int = np.array([0,0,1])		
		NORMAL_int = np.dot(T_inverse,NORMAL_int)		# Intersection plane normal
	
 	if axis == 5:
		X = np.linspace(Start,End,numPts)
		Y = np.linspace(Start1,End1,numPts1)		
		Xmesh, Ymesh = np.meshgrid(X, Y)
		Zmesh = 0*Xmesh
		POINTS = algs.make_vector(Xmesh.ravel(order='F'), Ymesh.ravel(order='F'), Zmesh.ravel(order='F'))
		normal = np.array([1,0,0])
		NORMAL = np.dot(T_inverse,normal)   			# Survey plane normal
		NORMAL_int = np.array([0,1,0])		
		NORMAL_int = np.dot(T_inverse,NORMAL_int)		# Intersection plane normal
		
	
############################################################################################################
#########################################  DISPLAY REFERENCE LINE  #########################################
############################################################################################################	

if Select_Action == 0:

	# Create the points along the line:
	pdo = self.GetPolyDataOutput()  												#Get a vtk.PolyData object for the output
	newPts = vtk.vtkPoints()  														#This will store the points for the line
	for i in range(0, numPts):   													#Generate the Points along the line
		newPts.InsertPoint(i, POINTS[i,0],POINTS[i,1],POINTS[i,2])   				#Insert the Points into the vtkPoints object
		
	pdo.SetPoints(newPts)  															#Add the points to the vtkPolyData object
	
	# Create the line from the points:
	ReferenceLine = vtk.vtkPolyLine()    											# This creates the line
	ReferenceLine.GetPointIds().SetNumberOfIds(numPts) 								#Indicate the number of points along the line
	
	for i in range(0,numPts):
	   ReferenceLine.GetPointIds().SetId(i, i)  									#Add the points to the line.

	   
	pdo.Allocate(1, 1) 																#Allocate the number of 'cells' that will be added. We are just adding one vtkPolyLine 'cell' to the vtkPolyData object.
	pdo.InsertNextCell(ReferenceLine.GetCellType(), ReferenceLine.GetPointIds())	#Add the poly line 'cell' to the vtkPolyData object.
	
	
############################################################################################################
########################################### DISPLAY SURVEY PLANE ###########################################
############################################################################################################

if  Output_Type == 0:
	if Select_Action == 1:

		input = self.GetInputDataObject(0, 0)  														# Reads input data

		plane = vtk.vtkPlane()																		# Creates slice plane
		plane.SetOrigin(POINTS[int(numPts/2),0], POINTS[int(numPts/2),1], POINTS[int(numPts/2),2])
		plane.SetNormal(NORMAL[0], NORMAL[1], NORMAL[2])

		cutter = vtk.vtkCutter()																	# Slices data with slice plane
		cutter.SetCutFunction(plane)
		cutter.SetInputData(input)
		cutter.Update()

		self.GetOutput().ShallowCopy(cutter.GetOutput())   											# Send sliced data to the output

		
############################################################################################################
########################################### DISPLAY MAPPING PLANE ##########################################
############################################################################################################
		
if  Output_Type == 1:
	if Select_Action == 1 or Select_Action == 2:
		
		# Send POINTS data to Paraview's output array:
		pts = vtk.vtkPoints()                                           	# create a vtkPoints container to store all the POINTS coordinates.
		pts.SetData(dsa.numpyTovtkDataArray(POINTS , "POINTS"))    			# numpyTovtkDataArray converts the NumPy array to a vtkDataArray, which is the format needed by SetData()
		output = self.GetOutput()                                       	# It calls the output port of the Paraview source
		output.SetPoints(pts)                                           	# It sends the points to the output
		#If needed, these output points can be visualized by applying a "Glyph" filter to the output of this programmable filter
		
	
	
		# Create surface mesh:
		NBcells=(numPts-1)*(numPts1-1)                         				# Number of cells
		
		IDlist = []
		for i in range(int(numPts)-1):                           			# We iterate over each cell in order to define the counter-clockwise point ID that defines the cell connectivity
			for j in range(int(numPts1)-1):
				IDlist.append([i*numPts1+j,j+(i+1)*numPts1,j+(i+1)*numPts1+1,i*numPts1+j+1])
		IDlist = np.array(IDlist)
			
		numCells = int(NBcells)
		output.Allocate(numCells, numPts*numPts1)                      		# Allocate the cells in the memory
		for cell in range(numCells):
			numCellPoints = int(4)                                          # There are 4 points for each cell (vtk_quad cell type)
			pointIds = vtk.vtkIdList()                                      # Creates an empty array to store the cells ID's
			for pointId in range(numCellPoints):
				pointIds.InsertId(pointId, int(IDlist[cell][pointId]))
			# cell type number is listed in vtkCellType.h
			# 12 is for hexahedron, 9 for VTK_quad
			output.InsertNextCell(9, pointIds)
	

############################################################################################################
#########################################  AUTOMATIC INTEGRATION  ##########################################
############################################################################################################

if Select_Action == 2:


	#############################################
	################ PASS ARRAYS ################
	#############################################
	
	#Use vtkPassArrays to let pass through the arrays of interest
	input = self.GetInputDataObject(0, 0)
	
	myArrays=vtk.vtkPassArrays()
	myArrays.SetInputDataObject(input)
	myArrays.ClearArrays()
	#myArrays.AddPointDataArray("AERO_Pt")

	for L in range(input.GetPointData().GetNumberOfArrays()):
		
		if Select_Variable == 0:  # it reads only the arrays corresponding to infinite surface formulations
		
			if input.GetPointData().GetArray(L).GetName().startswith('ARNTZ')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	
				
			if input.GetPointData().GetArray(L).GetName().startswith('MOMENTUM')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())			

			if input.GetPointData().GetArray(L).GetName().startswith('VDV')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	
				
			if input.GetPointData().GetArray(L).GetName().startswith('AGUIRRE_INF_')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	
				
		
		if Select_Variable == 1:  # it reads only the arrays corresponding to wake surface formulations

			if input.GetPointData().GetArray(L).GetName().startswith('ARNTZ_A')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
		
			if input.GetPointData().GetArray(L).GetName().startswith('AGUIRRE')==True:
				if input.GetPointData().GetArray(L).GetName().startswith('AGUIRRE_AERO')!=True:
					myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())

			if input.GetPointData().GetArray(L).GetName().startswith('BETZ')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	

			if input.GetPointData().GetArray(L).GetName().startswith('JONES')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	
				
			if input.GetPointData().GetArray(L).GetName().startswith('OSWATITSCH')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())

			if input.GetPointData().GetArray(L).GetName().startswith('MASKELL')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('GILES')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	

			if input.GetPointData().GetArray(L).GetName().startswith('KUSUNOSE')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	

			if input.GetPointData().GetArray(L).GetName().startswith('MEHEUT')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())				
					
			if input.GetPointData().GetArray(L).GetName().startswith('ONORATO')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('MELE')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('WU')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
					
		if Select_Variable == 2:  # it reads only the arrays corresponding to volume formulations

			if input.GetPointData().GetArray(L).GetName().startswith('ARNTZ_Aphi')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
			
			if input.GetPointData().GetArray(L).GetName().startswith('ARNTZ_At')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())

			if input.GetPointData().GetArray(L).GetName().startswith('ARNTZ_Aw')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('AGUIRRE_Aw')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('MELE_induced')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('MELE_lift')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('WU_induced')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())	

			if input.GetPointData().GetArray(L).GetName().startswith('WU_lift')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())				
			
		if Select_Variable == 3:  # it reads only the arrays corresponding to the output data from "Epsilon_Near_Field" filter
			myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())		
				
		if Select_Variable == 4:  # it reads only the arrays corresponding to the output data from "Epsilon_Profile_Drag_Breakdown" filter
			myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
			
		if Select_Variable == 5:  # it reads only the arrays: Maskell_lift and Maskell_sideforce
			if input.GetPointData().GetArray(L).GetName().startswith('MASKELL_lift')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
			if input.GetPointData().GetArray(L).GetName().startswith('MASKELL_sideforce')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
		
		if Select_Variable == 6:  # it reads only the "AERO" arrays from "Epsilon_Aero_Data" filter

			if input.GetPointData().GetArray(L).GetName().startswith('AERO')==True:	
				myArrays.AddPointDataArray(input.GetPointData().GetArray(L).GetName())
				
	myArrays.AddCellDataArray("")
	myArrays.AddFieldDataArray("") 
	myArrays.Update() 

	
	#############################################
	############ SPLIT VECTOR ARRAYS ############
	#############################################
	
	# Detect vectors and splits it into three different variables:  
	for L in range(myArrays.GetOutput().GetPointData().GetNumberOfArrays()):
	
		if myArrays.GetOutput().GetPointData().GetArray(L).GetNumberOfComponents() == 3:
			
			name = myArrays.GetOutput().GetPointData().GetArray(L).GetName()
			name_x = name+"_x"
			name_y = name+"_y"
			name_z = name+"_z"

			comp_x = vtk.vtkDoubleArray();
			comp_x.SetNumberOfComponents(1);
			comp_x.SetName(name_x);

			comp_y = vtk.vtkDoubleArray();
			comp_y.SetNumberOfComponents(1);
			comp_y.SetName(name_y);
			
			comp_z = vtk.vtkDoubleArray();
			comp_z.SetNumberOfComponents(1);
			comp_z.SetName(name_z);

			SSS = myArrays.GetOutput().GetPointData().GetArray(L).GetSize()	
			CCC = myArrays.GetOutput().GetPointData().GetArray(L).GetNumberOfComponents()
			ArrayLenght = SSS/CCC

			for K in range(ArrayLenght):
				Cx = myArrays.GetOutput().GetPointData().GetArray(L).GetComponent(K,0)
				Cy = myArrays.GetOutput().GetPointData().GetArray(L).GetComponent(K,1)
				Cz = myArrays.GetOutput().GetPointData().GetArray(L).GetComponent(K,2)
				
				comp_x.InsertNextValue(Cx);
				comp_y.InsertNextValue(Cy);
				comp_z.InsertNextValue(Cz);

			myArrays.GetOutput().GetPointData().AddArray(comp_x)
			myArrays.GetOutput().GetPointData().AddArray(comp_y)
			myArrays.GetOutput().GetPointData().AddArray(comp_z)
			
	myArrays.Update() 
	
	
	#############################################
	############# DISTRUBUTION LINE #############
	#############################################
	
	if  Output_Type == 0:   
				
		# Create points and line
		pdo = self.GetPolyDataOutput()  										#Get a vtk.PolyData object for the output
		newPts = vtk.vtkPoints()  												#This will store the points for the line

		Output_Line = vtk.vtkPolyLine()    										# This creates the line
		Output_Line.GetPointIds().SetNumberOfIds(numPts)
		
		vals = []
		for i in range(0, numPts):   											#Generate the Points along the line

			# Point coordinates:
			newPts.InsertPoint(i, POINTS[i,0],POINTS[i,1],POINTS[i,2])   		#Insert the Points into the vtkPoints object
			pdo.SetPoints(newPts)  												#Add the points to the vtkPolyData object (GetPolyDataOutput)
			Output_Line.GetPointIds().SetId(i, i)  								#Add the points to the line.
				
			# Create survey plane:
			plane = vtk.vtkPlane()
			plane.SetOrigin(POINTS[i,0],POINTS[i,1],POINTS[i,2])
			plane.SetNormal(NORMAL[0], NORMAL[1], NORMAL[2])

			
			if Select_Variable == 2 or Select_Variable == 5:
				# Creates a clip of the input with the survey plane:
				#cutter = vtk.vtkPVClipDataSet()
				cutter = vtk.vtkClipDataSet()
				cutter.SetClipFunction(plane)
				cutter.SetInputData(myArrays.GetOutput())
				cutter.InsideOutOn()
				cutter.Update()
			else:
				# Creates a slice of the input with the survey plane:
				cutter = vtk.vtkCutter()
				cutter.SetCutFunction(plane)
				cutter.SetInputData(myArrays.GetOutput())
				cutter.Update()
			
			# Integrate variables on the slice:
			integrate = vtk.vtkIntegrateAttributes()
			integrate.SetInputData(cutter.GetOutput())
			integrate.SetDivideAllCellDataByVolume(1)
			integrate.Update()

			integrated = integrate.GetOutput()
			pd = integrated.GetPointData()
			cd = integrated.GetCellData()
			
			# Add name and values to the output variables:
			for j in range(pd.GetNumberOfArrays()):
				if i == 0:
					arrayName = pd.GetArray(j).GetName()
					vals.append(vtk.vtkDoubleArray())
					vals[j].SetName(arrayName)
				
				arrayValue = pd.GetArray(j).GetTuple(0)[0]
				vals[j].InsertNextValue(arrayValue)
				
		for array in vals:
			pdo.GetPointData().AddArray(array)

		pdo.Allocate(1, 1) 															#Allocate the number of 'cells' that will be added. We are just adding one vtkPolyLine 'cell' to the vtkPolyData object.
		pdo.InsertNextCell(Output_Line.GetCellType(), Output_Line.GetPointIds())	#Add the poly line 'cell' to the vtkPolyData object.

		
	#############################################
	############## 3D TO 2D MAPPING #############
	#############################################
	
	if  Output_Type == 1:	
		
		vals = []
		pdo = self.GetOutput()
		#pdo = self.GetPolyDataOutput()  										#Get a vtk.PolyData object for the output
		for i in range(0, numPts): 
					
			# Create survey plane:
			plane = vtk.vtkPlane()
			plane.SetOrigin(POINTS[i*numPts1,0],POINTS[i*numPts1,1],POINTS[i*numPts1,2])
			plane.SetNormal(NORMAL[0], NORMAL[1], NORMAL[2])
		
			# Creates survey plane data: a slice of the input with the survey plane:
			survey = vtk.vtkCutter()
			survey.SetCutFunction(plane)
			survey.SetInputData(myArrays.GetOutput())
			survey.Update()		
				
			for j in range(0, numPts1):
							
				# Create intersection plane:
				plane_int = vtk.vtkPlane()
				plane_int.SetOrigin(POINTS[i*numPts1+j,0],POINTS[i*numPts1+j,1],POINTS[i*numPts1+j,2])
				plane_int.SetNormal(NORMAL_int[0], NORMAL_int[1], NORMAL_int[2])
				
				# Creates intersection line between the survey plane and the intersection plane
				int_line = vtk.vtkCutter()
				int_line.SetCutFunction(plane_int)
				int_line.SetInputData(survey.GetOutput())
				int_line.Update()		
		
				# Integrate variables along the line:
				integrate = vtk.vtkIntegrateAttributes()
				integrate.SetInputData(int_line.GetOutput())
				integrate.SetDivideAllCellDataByVolume(1)
				integrate.Update()

				integrated = integrate.GetOutput()
				pd = integrated.GetPointData()
				cd = integrated.GetCellData()

				# Add name and values to the output variables:
				for k in range(pd.GetNumberOfArrays()):
					if i == 0 and j == 0 :
						arrayName = pd.GetArray(k).GetName()
						vals.append(vtk.vtkDoubleArray())
						vals[k].SetName(arrayName)				
					
					arrayValue = pd.GetArray(k).GetTuple(0)[0]
					vals[k].InsertNextValue(arrayValue)
			
		for array in vals:
			pdo.GetPointData().AddArray(array)
		

############################################################################################################
###############################################  END OF CODE  ##############################################
############################################################################################################