# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################                 Near Field Plugin                #############################
############################################################################################################

# Abstract:
This Paraview's programmable filter takes 1 input from pipeline (it must be a SURFACE DATA) and calculates
the near-field aerodynamic parameters. Details of the formulations and how to use the plugin are available
in the User's Manual.


# Disclaimer:
This plugin was developed for research purposes and released under GNU GPL V3 license without any guarantee
of any kind, either express or implied. Its use for teaching, research and industrial applications is allowed
under the strict responsibility of the user. The authors of this plugin are not responsible for any damages
or errors related to the use of the plugin.
"""


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import vtk
from vtk.numpy_interface import algorithms as algs
from vtk.numpy_interface import dataset_adapter as dsa
import numpy as np

############################################################################################################
############################################# INPUT PARAMETERS #############################################
############################################################################################################

# Input parameters are defined as string in Paraview's GUI, thus a string to float convertion is required:
FlipNormals = float(FlipNormals)
Cref = float(Cref)
X0 = float(X0)     						# Moment center X-coordinate in the mesh reference frame
Y0 = float(Y0)     						# Moment center Y-coordinate in the mesh reference frame
Z0 = float(Z0)      					# Moment center Z-coordinate in the mesh reference frame

############################################################################################################
############################################## READ INPUT DATA #############################################
############################################################################################################

AdimF = inputs[0].FieldData["AdimF"]
DimF = inputs[0].FieldData["DimF"]
Sref = inputs[0].FieldData["Sref"]
TRANSFORMATION = inputs[0].FieldData["TRANSFORMATION"]
Cp = inputs[0].PointData["AERO_Cp"]
Ps = inputs[0].PointData["AERO_Ps"]
Tau = inputs[0].PointData["AERO_Stress(tau)"]
MU_LAM = inputs[0].PointData["AERO_MU_LAM"]
MU_EFF = inputs[0].PointData["AERO_MU_EFF"]
Vorticity = inputs[0].PointData["AERO_Vorticity"]
V_gradient = inputs[0].PointData["AERO_V_gradient"]
V = inputs[0].PointData["AERO_Vabs"]
AERO_Vector_Speed = inputs[0].PointData["AERO_Vector_Speed"]
POINTS = inputs[0].Points


############################################################################################################
############################################# NEAR-FIELD METHOD ############################################
############################################################################################################

if FlipNormals==0:   # The user must be sure that normals are pointing outward the body surface
 FN = 1
else:
 FN = -1

NORMALS = FN*vertex_normal(inputs[0])  							# NORMALS vector in the mesh reference frame
normals = np.einsum('ij,kj->ik',NORMALS,TRANSFORMATION) 		# normals vector in the aerodynamic reference frame (einsum=Pythonic way to calculate the dot product)


F_pressure = -1*Cp*normals*DimF/(Sref*AdimF)
#F_pressure = -1*Ps*normals/AdimF   # Works OK in 2D but not OK in 3D. Expression left here for debuging purposes

# In the following expressions, a factor of 2 was included because ParaView struggles to compute derivative at walls properly
# The derivative calculation gives but the good values for the nodes inside the fluid domain but half values at the wall

F_friction = -2*algs.cross(normals,MU_LAM*Vorticity)/AdimF       	# Wu equation 18 (1989)

# Alternative Wall Shear Stress Expressions: left here for testing purposes
#F_friction2 = 2*np.einsum('ij,ikj->ik',normals,2*Tau)/AdimF     	# Shear stress = 2*mu*strain = 2*Tau 	
#F_friction3 = 2*(MU_LAM*(V_gradient[:,0,1]*normals[:,0]+V_gradient[:,0,2]*normals[:,0]+V_gradient[:,1,0]*normals[:,1]+V_gradient[:,2,0]*normals[:,2]) ) /AdimF # X-component WSS by definition
#F_friction4 = 2*(normals[:,0]*MU_LAM*(2*V_gradient[:,0,0]) + normals[:,1]*MU_LAM*(V_gradient[:,0,1]+V_gradient[:,1,0]) + normals[:,2]*MU_LAM*(V_gradient[:,0,2]+V_gradient[:,2,0]))/AdimF  # X-component WSS 


F_total = F_pressure+F_friction



if AdimF==1:
	AdimM = 1
else:
	AdimM = Cref
	

COORDS = algs.make_vector(POINTS[:,0]-X0,POINTS[:,1]-Y0,POINTS[:,2]-Z0)
coords = np.einsum('ij,kj->ik',COORDS,TRANSFORMATION)
M_pressure = algs.cross(coords,F_pressure)/(AdimM)  # Since force has been already non-dimensionalized, here only the distances are non-dimensionalized
M_friction = algs.cross(coords,F_friction)/(AdimM)
M_total = M_pressure+M_friction 


Vorticity_wall = 2*Vorticity        # Wall-corrected vorticity
V_gradient_wall = 2*V_gradient      # Wall-corrected velocity gradients


############################################################################################################
################################################ OUTPUT DATA ###############################################
############################################################################################################

output.PointData.append(normals,"normals")
output.PointData.append(Ps, "AERO_Ps")
output.PointData.append(Cp,"Cp")
output.PointData.append(Vorticity_wall,"Vorticity_wall")
output.PointData.append(V_gradient_wall,"V_gradient_wall")
output.PointData.append(F_total,"F_total")
output.PointData.append(F_pressure,"F_pressure")
output.PointData.append(F_friction,"F_friction")
output.PointData.append(M_total,"M_total")
output.PointData.append(M_pressure,"M_pressure")
output.PointData.append(M_friction,"M_friction")
output.PointData.append(AERO_Vector_Speed,"AERO_Vector_Speed")




