# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################               Poisson Solver Plugin              #############################
############################################################################################################

# Abstract:
This Paraview's programmable filter takes a slice data from Epsilon_Aero_Data filter and solves the Poisson 
equation to find both, the stream function and velocity potential function. This functions allows then to 
calculate the vortex drag density on each point of the mesh plane. The user has to indicate if he wants to
calculate or not the velocity potential (and its contribution to the vortex drag). By default it is not 
considered. The Poisson equation is solved by using Green functions: it is an analytical solution wich can be 
used in all types of mesh distributions. It allows to choose a free field solution for full models and half 
models (symmetry condition).

"""




# ATTENTION: this filter requires the "Near Field" option selected in Epsilon_Aero_Data !!!!!!



############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import time
import numpy as np
import vtk
import vtk.numpy_interface.dataset_adapter as dsa
from vtk.numpy_interface import algorithms as algs
import paraview.vtk.util.numpy_support as vnp
import numpy.ma as ma


############################################################################################################
############################################# STRING TO FLOAT ##############################################
############################################################################################################

# Input parameters are defined as string in Paraview's GUI, thus a string to float convertion is required:
EnablePhi = float(EnablePhi)   						# Velocity potential on the induced drag: 0=not activated  1=activated
SigmaDefinition = float(SigmaDefinition)  			# Choose the definition of Sigma: 0=-du_dx  1=dv_dy+dw_dz
HalfModel = float(HalfModel)       					# Green functions:  0=half model geometry   1=full model geometry


############################################################################################################
########################################## READING CFD INPUT DATA ##########################################
############################################################################################################

y = inputs[0].Points[:,1]                                       # Y-coordinates       
z = inputs[0].Points[:,2]                                       # Z-coordinates 
Zita = inputs[0].PointData["AERO_Vorticity"][:,0]         		# x-vorticity (aero reference frame)
Vector_Speed = inputs[0].PointData["AERO_Vector_Speed"]         # Velocity vector in the mesh reference frame
du_dx = inputs[0].PointData["AERO_V_gradient"][:,0,0]          	# x-velocity gradient along x-axis (wind axes)
dv_dy = inputs[0].PointData["AERO_V_gradient"][:,1,1]          	# y-velocity gradient along y-axis (wind axes)
dw_dz = inputs[0].PointData["AERO_V_gradient"][:,2,2]          	# z-velocity gradient along z-axis (wind axes)
u = inputs[0].PointData["AERO_u"]                               # x-velocity component
V0 = inputs[0].FieldData["V0"]                                	# Upstream velocity 
Pt = inputs[0].PointData["AERO_Pt"]                             # Total pressure
Pt0 = inputs[0].FieldData["Pt0"]                               	# Total pressure upstream
Tt = inputs[0].PointData["AERO_Tt"]                             # Total temperature
Tt0 = inputs[0].FieldData["Tt0"]                               	# Total temperature upstream
Rho = inputs[0].PointData["AERO_Rho"]                           # Density
Rho0 = inputs[0].FieldData["Rho0"]                              # Upstream density
M0 = inputs[0].FieldData["M0"]                                	# Upstream Mach number
AdimF = inputs[0].FieldData["AdimF"]                            # Force adimensionalization factor 
AdimP = inputs[0].FieldData["AdimP"]                            # Power adimensionalization factor 
UpwardsAxis = inputs[0].FieldData["UpwardsAxis"]   				# 0=Y-axis   1=Z-axis
TRANSFORMATION = inputs[0].FieldData["TRANSFORMATION"]   		# Transformation matrix

############################################################################################################
########################################## PRELIMINARY CALCULATION #########################################
############################################################################################################

Gamma=1.4


############################################################################################################
######################################### SOLVING POISSON EQUATION #########################################
############################################################################################################

print "Starting Poisson solver..."
start = time.time()                                             # This is ised to measure the required time to solve the Poisson equation


##############################################
########### GREEN FUNCTIONS SOLVER ###########
##############################################

# Calculates cells area
area = algs.area(inputs[0])   						

# Find cell centers
pdi = self.GetInputDataObject(0, 0)  				
cen = vtk.vtkCellCenters()
cen.SetInputDataObject(pdi)
cen.Update()
centers = dsa.WrapDataObject(cen.GetOutput()).Points


# Converting vorticity and sigma arrays from Point Data To Cell Data:
myArrays=vtk.vtkPassArrays()                                 	# use pass arrays to extract a copy of the input data
myArrays.SetInputDataObject(self.GetInputDataObject(0,0))       # We call the Paraview internal port where input data is stored
myArrays.ClearArrays()
myArrays.AddPointDataArray('AERO_Vorticity')
myArrays.AddPointDataArray('AERO_V_gradient')
myArrays.AddCellDataArray('')
myArrays.AddFieldDataArray('')
myArrays.Update()


p2c = vtk.vtkPointDataToCellData()
p2c.SetInputConnection(myArrays.GetOutputPort())
p2c.Update()
#self.GetOutput().ShallowCopy(p2c.GetOutput())   				# Retained here for debug purposes

INPUT = dsa.WrapDataObject(p2c.GetOutput()) 					# Get the cell data
Zita_cell = INPUT.CellData["AERO_Vorticity"][:,0]
V_grad_cell = INPUT.CellData["AERO_V_gradient"]


# Calculating the source term of the incompressible continuity equation:
if SigmaDefinition==0:
	Sigma = - du_dx
	Sigma_cell = - V_grad_cell[:,0,0]
else:
	Sigma = dv_dy + dw_dz
	Sigma_cell =  V_grad_cell[:,1,1]+V_grad_cell[:,2,2]

	
# Green function implementation:
Psi  = np.zeros(Zita.size)   # Stream function initialized as zero everywhere
Phi  = np.zeros(Zita.size)   # Potential function initialized as zero everywhere    



for i in range(np.size(Psi)):     # Iterates at each point of the mesh and calculates there the Green function. This function changes depending if the wake data correspond to a half model or full model
	
	if HalfModel==0:  # For half models, the Green function takes into account the mirror domain
		if UpwardsAxis==0:
			Logarithm = ((y[i]-centers[:,1])**2+(z[i]-centers[:,2])**2) / ((y[i]-centers[:,1])**2+(z[i]+centers[:,2])**2)

		elif UpwardsAxis==1:
			Logarithm = ((y[i]-centers[:,1])**2+(z[i]-centers[:,2])**2) / ((y[i]+centers[:,1])**2+(z[i]-centers[:,2])**2)
	
	elif HalfModel==1:
		Logarithm = (y[i]-centers[:,1])**2+(z[i]-centers[:,2])**2
	
	Psi[i] =  np.sum( -Zita_cell[:]*area[:]*(0.25/np.pi)*np.log(Logarithm[:]))
	
	if EnablePhi==1:
		Phi[i] =  np.sum( Sigma_cell[:]*area[:]*(0.25/np.pi)*np.log(Logarithm[:]))        


# Printing the time required to solve the Poisson equation:
end = time.time()
SolveTime = end - start
print "Calculation time", SolveTime, " Seconds (", SolveTime/60.0, " minutes)"  
print ""


############################################################################################################
########################################## VORTEX DRAG CALCULATION #########################################
############################################################################################################

# Maskell method:
MASKELL_vortex_drag = 0.5*Rho0*(Psi*Zita-Phi*Sigma)/AdimF

# Aguirre method:
ETA = 1+0.5*(Gamma-1)*M0**2*(1-((u**2+Psi*Zita-Phi*Sigma)*Tt0/(V0**2*Tt)))
Raiz = 1- ((ETA*(Pt/Pt0)**((Gamma-1)/Gamma)-1)/(0.5*(Gamma-1)*M0**2)) - ((Psi*Zita-Phi*Sigma)/V0**2)            # Term inside the root of DeltaUstar
Ustar = V0*algs.sqrt(Raiz)                                                                                      # Deficit of velocity related to the potential effect
Rhostar = Rho0*(1+0.5*(Gamma-1)*M0**2*(1-((u**2+Psi*Zita-Phi*Sigma)/V0**2)))**(1/(Gamma-1))           			# Density related to the potential effect
AGUIRRE_vortex_exergy = 0.5*Rho*(Psi*Zita-Phi*Sigma)*Ustar/AdimP

where_are_NaNs = isnan(AGUIRRE_vortex_exergy)    # Sometimes the ResampleWithDataSet fails to set data in some points, leading to NaN value in Ev_Maskell, avoiding its integration on the survey plane. This eliminates this problem.
AGUIRRE_vortex_exergy[where_are_NaNs] = 0

"""
# Induced velocities:
TRANS_T = np.transpose(TRANSFORMATION)										# M transpose (M=transformation matrix)

aaaaa = vnp.numpy_to_vtk(Psi)
INPUTTTTTT = dsa.WrapDataObject(aaaaa)
Psi_gradient_C = gradient(Psi)   											# Gradient of Psi in the mesh reference axes
Psi_gradient_A = algs.gradient(Psi) 


TRANS_Psi_gradient = np.einsum('ijm,kj->ikm',Psi_gradient,TRANSFORMATION)  	# Matrix product: M.Psi_gradient   
Psi_gradient = np.einsum('km,ijk->ijm',TRANS_T,TRANS_Psi_gradient)     		# Psi gradient in aero reference fram (tensor transformation: M.Psi_gradient.Mtranspose)
v_Psi = Psi_gradient[:,2]   												# induced y-velocity component 
w_Psi = -Psi_gradient[:,1]													# induced z-velocity component 

Phi_gradient = np.gradient(Phi)												# Gradient of Phi in the mesh reference axes
TRANS_Phi_gradient = np.einsum('ijm,kj->ikm',Phi_gradient,TRANSFORMATION)  	# Matrix product: M.Phi_gradient   
Phi_gradient = np.einsum('km,ijk->ijm',TRANS_T,TRANS_Phi_gradient)     		# Phi gradient in aero reference fram (tensor transformation: M.Phi_gradient.Mtranspose)
v_Phi = Phi_gradient[:,1]   												# induced y-velocity component 
w_Phi = Phi_gradient[:,2]													# induced z-velocity component 

induced_v = v_Psi + v_Phi
induced_w = w_Psi + w_Phi
"""


############################################################################################################
###############################################  DATA OUTPUT ###############################################
############################################################################################################

output.PointData.append(Vector_Speed, "Vector_Speed")
output.PointData.append(Psi , 'Stream_Function')
output.PointData.append(Phi , 'Potential_Function')
output.PointData.append(Sigma, "Sigma")
output.PointData.append(inputs[0].PointData["Vorticity_x"], "Vorticity_x")
output.PointData.append(MASKELL_vortex_drag , 'MASKELL_vortex_drag')
output.PointData.append(AGUIRRE_vortex_exergy, "AGUIRRE_vortex_exergy")
output.PointData.append(Phi*Sigma, "Phi*Sigma")
#output.PointData.append(induced_v, "induced_v")
#output.PointData.append(induced_w, "induced_w")






