# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################               Drag Breakdown Plugin              #############################
############################################################################################################

Contact : Miguel Angel AGUIRRE (Miguel-angel.AGUIRRE@isae-supaero.fr)

# Abstract:
This Paraview's programmable filter performs the breakdown of the profile drag into its viscous, wave and jet
components. It takes the desired variable from the Epsilon_Aero_Data output and it  looks for negative drag 
points which are tagged as "jet" regions. Then the wave drag regions are detected by knowing that vorticity 
and drag are weak there. Finally the viscous drag is extracted. 
"""


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import numpy as np
import vtk
from vtk.numpy_interface import algorithms as algs


############################################################################################################
############################################# INPUT PARAMETERS #############################################
############################################################################################################

# The following parameters are already defined in Paraview's GUI, but they are kept here for debugging purposes:

#DragThreshold = 0.25                                                       # Percentage of the maximum drag density (0 to 1 range value)
#VorticityThreshold = 0.02                                                  # Percentage of the maximum vorticity (0 to 1 range value)


############################################################################################################
############################################# STRING TO FLOAT ##############################################
############################################################################################################

# Input parameters are defined as string in Paraview's GUI, thus a string to float convertion is required:
DragThreshold = float(DragThreshold) 
VorticityThreshold = float(VorticityThreshold) 
Absolute_Threshold = float(Absolute_Threshold)					# 1=Thresholds are absolute, 0=Thresholds are relative (Percentage of the maximum vorticity: 0 to 1 range value)
Breakdown_Exergy = float(Breakdown_Exergy)						# 0=Uses vorticity for wave detection   1=Uses viscous anergy for wave detection

############################################################################################################
############################################ READING INPUT DATA ############################################
############################################################################################################

# Reading data from the input:
Profile_Drag = inputs[0].PointData[Parameter]   					# "Parameter" is the aerodynamic variable selected by the user in the GUI (Some profile drag output from Epsilon_Aero_Data)
AERO_Vorticity = inputs[0].PointData["AERO_Vorticity"]

if Breakdown_Exergy == 1:
	ARNTZ_Aphi = inputs[0].PointData["ARNTZ_Aphi"]

# Eliminating NaN values:
where_are_NaNs = isnan(Profile_Drag)    # Sometimes Profile_Drag contains NaN value in the array, avoiding the detection of its maximum (required later). This eliminates this problem.
Profile_Drag[where_are_NaNs] = 0

where_are_NaNs = isnan(AERO_Vorticity)    
AERO_Vorticity[where_are_NaNs] = 0

if Breakdown_Exergy == 1:
	where_are_NaNs = isnan(ARNTZ_Aphi)    
	ARNTZ_Aphi[where_are_NaNs] = 0

Vorticity_Magnitude = algs.mag(AERO_Vorticity)


############################################################################################################
######################################### BREAKDOWN OF PROFILE DRAG ########################################
############################################################################################################

# DETECTING JET REGIONS:
# It look for the regions where the profile drag is negative. The points not satisfying this condition are masked (settled to zero)
MASK= Profile_Drag < 0.0
Jet = 1*MASK*Profile_Drag

if Breakdown_Exergy == 0:

	# DETECTING WAVE DRAG REGIONS:
	# It looks for regions with weak vorticity and weak profile drag. The points not satisfying this condition are masked (settled to zero)
	if Absolute_Threshold == 0:
		Cut_Value_Drag = DragThreshold*algs.max(Profile_Drag)
		Cut_Value_Vorticity = VorticityThreshold*algs.max(Vorticity_Magnitude)
	else:
		Cut_Value_Drag = DragThreshold
		Cut_Value_Vorticity = VorticityThreshold

	MASK1= 0.0 < Profile_Drag  
	MASK2 = Profile_Drag < Cut_Value_Drag
	MASK3 = Vorticity_Magnitude < Cut_Value_Vorticity
	wave = 1*MASK1*MASK2*MASK3*Profile_Drag

	# DETECTING VISCOUS DRAG REGIONS:
	# It looks for regions with high profile drag. The points not satisfying this condition are masked (settled to zero)
	MASK4 = Profile_Drag > Cut_Value_Drag
	viscous = 1*MASK1*MASK4*Profile_Drag

if Breakdown_Exergy == 1:
	
	# DETECTING WAVE DRAG REGIONS:
	# It looks for regions with weak viscous anergy and weak profile drag. The points not satisfying this condition are masked (settled to zero)
	if Absolute_Threshold == 0:
		Cut_Value_Drag = DragThreshold*algs.max(Profile_Drag)
		Cut_Value_Aphi = VorticityThreshold*algs.max(ARNTZ_Aphi)
	else:
		Cut_Value_Drag = DragThreshold
		Cut_Value_Aphi = VorticityThreshold
	
	MASK1= 0.0 < Profile_Drag 
	MASK2 = Profile_Drag < Cut_Value_Drag 
	MASK3 = ARNTZ_Aphi < Cut_Value_Aphi
	wave = 1*MASK1*MASK2*MASK3*Profile_Drag 
	
	# DETECTING VISCOUS DRAG REGIONS:
	# It looks for regions with high profile drag. The points not satisfying this condition are masked (settled to zero)
	MASK4 = Profile_Drag > Cut_Value_Drag 
	viscous = 1*MASK1*MASK4*Profile_Drag



############################################################################################################
###############################################  DATA OUTPUT ###############################################
############################################################################################################

# Basic aero data:
output.PointData.append(inputs[0].PointData["Points"], "Points")
output.PointData.append(Vorticity_Magnitude, "Vorticity_Magnitude")
output.PointData.append(Profile_Drag, "Profile_Drag")
output.PointData.append(viscous, "viscous")
output.PointData.append(wave, "wave")
output.PointData.append(Jet, "Jet")

if Breakdown_Exergy == 1:
	output.PointData.append(ARNTZ_Aphi, "ARNTZ_Aphi")


	
	








