# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################                Wind Tunnel Plugin                #############################
############################################################################################################


# Abstract:
This Paraview's programmable source allows creating a delaunay2D mesh from a PIV or multi-hole probe CSV data file 
"""


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

import vtk
import numpy as np


############################################################################################################
############################################# STRING TO FLOAT ##############################################
############################################################################################################

# Input parameters are defined as string in Paraview GUI, thus a string to float convertion is required:

File_Path = str(File_Path)  			# File path to the CSV data
Instrument = float(Instrument)  		# 0=PIV 2D2C/stereo   1=multi-hole probe
Survey_Plane = float(Survey_Plane) 		# 0=YZ plane	1=XZ plane	2=XY plane
Isoenergetic = str(Isoenergetic)

# The following parameters are defined as a vector in Paraview, so each component is assigned to a separate variable:
X0 = Origin[0]  	# X-coordinate of the axes origin
Y0 = Origin[1]
Z0 = Origin[2]


############################################################################################################
############################################## READ INPUT DATA #############################################
############################################################################################################


data = np.genfromtxt(File_Path, dtype=None, names=True, delimiter=",", autostrip=True)

X = data["X"] 
Y = data["Y"] 
Z = data["Z"] 
Vx = data["Vx"]
Vy = data["Vy"]
Vz = data["Vz"]

if Instrument == 1:
	Ps = data["Ps"]
	Pt = data["Pt"]

############################################################################################################
############################################## CREATING PLANE ##############################################
############################################################################################################

# PLANE ORIENTATION DEFINITION: it adapts the input coordinates for the Delaunay2D (that can only be created in the xy plane)
if Survey_Plane == 0:
	x, y, z =  Y, Z, X
	
if Survey_Plane == 1:
	x, y, z =  X, Z, Y
	
if Survey_Plane == 2:
	x, y, z =  X, Y, Z

# DELAUNAY PLANE CREATION:
points = vtk.vtkPoints()  						# Creates an array to store the points
 
for i in range(X.size):
    points.InsertNextPoint(x[i],y[i],z[i])
 
aPolyData = vtk.vtkPolyData()
aPolyData.SetPoints(points)
 
aCellArray = vtk.vtkCellArray()
 
boundary = vtk.vtkPolyData()
boundary.SetPoints(aPolyData.GetPoints())
boundary.SetPolys(aCellArray)
delaunay = vtk.vtkDelaunay2D()
if vtk.VTK_MAJOR_VERSION <= 5:
    delaunay.SetInput(aPolyData.GetOutput())
    delaunay.SetSource(boundary)
else:
    delaunay.SetInputData(aPolyData)
    delaunay.SetSourceData(boundary)

delaunay.Update()
#self.GetOutput().ShallowCopy(delaunay.GetOutput())


# DELAUNAY PLANE TRANSFORMATION:
rX = vtk.vtkTransform()  		# Rotation about X
rX.RotateX(90)
rX.Update()

rY = vtk.vtkTransform() 		# Rotation about Y
rY.RotateY(90)
rY.Update()

rZ = vtk.vtkTransform()			# Rotation about Z
rZ.RotateZ(90)
rZ.Update()

trans1 = vtk.vtkTransform() 	# Translation
trans1.Translate(X0, Y0, Z0)
trans1.Update()


if Survey_Plane == 0:

	trans_filter1 = vtk.vtkTransformFilter()
	trans_filter1.SetTransform(rX)
	trans_filter1.SetInputData(delaunay.GetOutput())
	trans_filter1.Update()

	trans_filter2 = vtk.vtkTransformFilter()
	trans_filter2.SetTransform(rZ)
	trans_filter2.SetInputData(trans_filter1.GetOutput())
	trans_filter2.Update()
	
	delaunay_new = trans_filter2
	
if Survey_Plane == 1:
	trans_filter1 = vtk.vtkTransformFilter()
	trans_filter1.SetTransform(rX)
	trans_filter1.SetInputData(delaunay.GetOutput())
	trans_filter1.Update()
	
	delaunay_new = trans_filter1
	
if Survey_Plane == 2:	
	delaunay_new = delaunay
	
trans_filter3 = vtk.vtkTransformFilter()
trans_filter3.SetTransform(trans1)
trans_filter3.SetInputData(delaunay_new.GetOutput())
trans_filter3.Update()

PLANE = trans_filter3

self.GetOutput().ShallowCopy(PLANE.GetOutput())




############################################################################################################
############################################### DATA OUTPUT ################################################
############################################################################################################

# First we define a function that adds point aero data to the output for the mesh:
numPoints = X.size                 			# Number of points of the  mesh

def addfielddata(numTuples, dataSetAttributes,DataArray,ArrayName): # Defines a function that adds the field data to the existing points in the Paraview's output
  numberOfComponents = int(1)                                       # We deal only with 1-component arrays
  dataArray = vtk.vtkDoubleArray()
  dataSetAttributes.AddArray(dataArray)
  dataArray.SetNumberOfComponents(numberOfComponents)
  dataArray.SetName(ArrayName)                                      # "ArrayName" is a string used to name the array in Paraview 
  data = DataArray                                                  # DataArray is the numpy array with the aerodynamic parameters that we want to associate to the points
  for value in data:
    dataArray.InsertNextValue(float(value))                         # Iterates along every value of the array and add its value to the output array designed with the "ArrayName" name 
 

# Now we use this function to send the scalar arrays to the output:
output = self.GetOutput()
addfielddata(numPoints, output.GetPointData(),Vx,"Vx")
addfielddata(numPoints, output.GetPointData(),Vy,"Vy")
addfielddata(numPoints, output.GetPointData(),Vz,"Vz")

if Instrument == 1:
	addfielddata(numPoints, output.GetPointData(),Ps,"Ps")
	addfielddata(numPoints, output.GetPointData(),Pt,"Pt")


addfielddata(numPoints, output.GetFieldData(),Isoenergetic,"Isoenergetic")


