# -*- coding: utf-8 -*-
"""
############################################################################################################
#############################                     EPSILON                      #############################
#############################             STAR CCM Ensight reader              #############################
############################################################################################################

# Abstract:
This python code is a module of the "Epsilon Aero Data" plugin that reads some variables from a STAR CCM file
exported into the Ensight Gold Case format.


# Disclaimer:
This plugin was developed for research purposes and released under GNU GPL V3 license without any guarantee
of any kind, either express or implied. Its use for teaching, research and industrial applications is allowed
under the strict responsibility of the user. The authors of this plugin are not responsible for any damages
or errors related to the use of the plugin.
"""


############################################################################################################
############################################### MODULES LOAD ###############################################
############################################################################################################

# We load the Python and Paraview standard modules:
import numpy as np
import vtk
from vtk.numpy_interface import algorithms as algs


############################################################################################################
##################################### STAR CCM ENSIGHT READER FUNCTION #####################################
############################################################################################################

def Reader_STAR_CCM_Ensight(INPUT,R,CvGas,CpGas,Ps0,Rho0,Ts0):


	# Reading aero data from Fluent file:
	POINTS = INPUT.Points   															# It takes the points coordinates
	Vx = INPUT.PointData["Velocityi"] 													# Speed component along x-axis [m/s]
	print ""
	
	# Read Velocityj data:
	if INPUT.GetPointData().HasArray("Velocityj")==0:                           		# This check if "Velocityj" array does not exists in the input                
		print "# STAR CCM: You are loading a 2D data (No Velocityj data available)"    	# It means that Velocityj array does not exists 
		Vy = 0*Vx                                                              			# If the array does not exists,it creates a null vector
	else:
		Vy = INPUT.PointData["Velocityj"]                                       		# If the array exists, then reads the array data directly from input
    
    # Read Velocityk data:
	if INPUT.GetPointData().HasArray("Velocityk")==0:                           		# This check if "Velocityk" array does not  exists in the input                
		print "# STAR CCM: You are loading a 2D data (No Velocityk data available)"  	# It means that Velocityk array does not exists 
		Vz = 0*Vx
	else:
		Vz = INPUT.PointData["Velocityk"]                                       		# If the array exists, then reads the array data directly from input
		
	# Read static pressure data:
	if INPUT.GetPointData().HasArray("AbsolutePressure")==0:                           	# This check if "AbsolutePressure" array does not  exists in the input 
		if algs.min(INPUT.GetPointData().GetArray("StaticPressure"))<0:           		# This check if "StaticPressure" array contains negative values
			print "# STAR CCM: Static pressure input is relative to atmosphere. A conversion to absolute static pressure is made"                         
			Ps = INPUT.PointData["StaticPressure"]+Ps0                            		# StaticPressure array is calculated from loaded data 
		else:
			Ps = INPUT.PointData["StaticPressure"]                                 		# If the array contains absolute pressure values, then it reads the array data directly from input
	else:
		Ps = INPUT.PointData["AbsolutePressure"]                                         
	
 	# Read density data:
	if INPUT.GetPointData().HasArray("Density")==0: 
		print "# STAR CCM: Density array is not available in the input data. An incompressible flow will be assumed" 
		Rho = Rho0*Vx/Vx      															#Vx/Vx creates a vector filled with "1"
	else:
		Rho = INPUT.PointData["Density"]                                				# Density [Kg/m^3]

	# Read static temperature data:
	if INPUT.GetPointData().HasArray("Temperature")==0:                           		# This check if "Temperature" array does not  exists in the input 
		if INPUT.GetPointData().HasArray("StaticTemperature")==0:                     	# This check if "StaticTemperature" array does not exists in the input 
			print "# STAR CCM: Static Temperature array is not available in the input data"          	# It means that "StaticTemperature" array does not exists (It may happen when energy equation is not activated in the RANS solution)
			print "# STAR CCM: Static Temperature data is reconstructed from static pressure and density arrays"
			Ts = Ps/(R*Rho)                                                                           	# "StaticTemperature" array is calculated from loaded data 
		else:
			Ts = INPUT.PointData["StaticTemperature"]                                 	# If the array exists, then reads the array data directly from input
	else:
		Ts = INPUT.PointData["Temperature"]     

	# Read viscosity data:
	MU_TURB = INPUT.PointData["TurbulentViscosity"]                 					# Turbulent viscosity [Kg/m*s]	
	
	if INPUT.GetPointData().HasArray("TurbulentViscosityRatio")==0:                  	# This check if "TurbulentViscosityRatio" array does not  exists in the input 	
		MU_EFF = INPUT.PointData["EffectiveViscosity"] 									# Effective viscosity [Kg/m*s] 
		MU_LAM = MU_EFF - MU_TURB														# Molecular viscosity [Kg/m*s]
	else:
		MU_ratio = INPUT.PointData["TurbulentViscosityRatio"]
		MU_LAM = MU_TURB/MU_ratio
		MU_EFF = MU_LAM + MU_TURB
		
	# Effective dissipation:
	Prt = 0.7     #IMPORTANT --> A STRONG HYPOTESIS IS MADE HERE (Pr=0.7) because no Prandtl number data available on STAR CCM file format
	Keff = CpGas*((MU_LAM/0.7)+(MU_TURB/Prt))

	return POINTS,Vx,Vy,Vz,Ps,Ts,Rho,MU_LAM,MU_TURB,MU_EFF,Keff